from pathlib import Path
from typing import List

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, DEFAULT_SOURCE_VIEW_NAME, DEFAULT_SEACROWD_VIEW_NAME
from seacrowd.utils.common_parser import load_conll_data

_DATASETNAME = "idn_tagged_corpus_csui"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False
_CITATION = """\
@inproceedings{dinakaramani2014designing,
  title={Designing an Indonesian part of speech tagset and manually tagged Indonesian corpus},
  author={Dinakaramani, Arawinda and Rashel, Fam and Luthfi, Andry and Manurung, Ruli},
  booktitle={2014 International Conference on Asian Language Processing (IALP)},
  pages={66--69},
  year={2014},
  organization={IEEE}
}

@inproceedings{kurniawan2018towards,
  author={Kurniawan, Kemal and Aji, Alham Fikri},
  booktitle={2018 International Conference on Asian Language Processing (IALP)}, 
  title={Toward a Standardized and More Accurate Indonesian Part-of-Speech Tagging}, 
  year={2018},
  volume={},
  number={},
  pages={303-307},
  doi={10.1109/IALP.2018.8629236}}
"""

_DESCRIPTION = """\
Idn-tagged-corpus-CSUI is a POS tagging dataset contains about 10,000 sentences, collected from the PAN Localization Project tagged with 23 POS tag classes.
The POS tagset is created through a detailed study and analysis of existing tagsets and the manual tagging of an Indonesian corpus.
Idn-tagged-corpus-CSUI dataset is splitted into 3 sets with 8000 train, 1000 validation, 1029 test data.
"""

_HOMEPAGE = "https://bahasa.cs.ui.ac.id/postag/corpus"

_LICENSE = "Creative Commons Attribution Share-Alike 4.0 International"

_URLs = {
    "train": "https://raw.githubusercontent.com/ir-nlp-csui/idn-tagged-corpus-CSUI/master/IndoNLU-split/train_preprocess.conll",
    "validation": "https://raw.githubusercontent.com/ir-nlp-csui/idn-tagged-corpus-CSUI/master/IndoNLU-split/valid_preprocess.conll",
    "test": "https://raw.githubusercontent.com/ir-nlp-csui/idn-tagged-corpus-CSUI/master/IndoNLU-split/test_preprocess.conll",
}

_SUPPORTED_TASKS = [Tasks.POS_TAGGING]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IdnTaggedCorpusCSUI(datasets.GeneratorBasedBuilder):
    """Idn-tagged-corpus-CSUI is a POS tagging dataset contains about 10,000 sentences, collected from the PAN Localization Project tagged with 23 POS tag classes."""
    label_classes=[
        "B-PR",
        "B-CD",
        "I-PR",
        "B-SYM",
        "B-JJ",
        "B-DT",
        "I-UH",
        "I-NND",
        "B-SC",
        "I-WH",
        "I-IN",
        "I-NNP",
        "I-VB",
        "B-IN",
        "B-NND",
        "I-CD",
        "I-JJ",
        "I-X",
        "B-OD",
        "B-RP",
        "B-RB",
        "B-NNP",
        "I-RB",
        "I-Z",
        "B-CC",
        "B-NEG",
        "B-VB",
        "B-NN",
        "B-MD",
        "B-UH",
        "I-NN",
        "B-PRP",
        "I-SC",
        "B-Z",
        "I-PRP",
        "I-OD",
        "I-SYM",
        "B-WH",
        "B-FW",
        "I-CC",
        "B-X",
    ]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="idn_tagged_corpus_csui_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="Idn-tagged-corpus-CSUI source schema",
            schema="source",
            subset_id="idn_tagged_corpus_csui",
        ),
        SEACrowdConfig(
            name="idn_tagged_corpus_csui_seacrowd_seq_label",
            version=datasets.Version(_SEACROWD_VERSION),
            description="Idn-tagged-corpus-CSUI Nusantara schema",
            schema="seacrowd_seq_label",
            subset_id="idn_tagged_corpus_csui",
        ),
    ]

    DEFAULT_CONFIG_NAME = "idn_tagged_corpus_csui_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features({"index": datasets.Value("string"), "tokens": [datasets.Value("string")], "pos_tag": [datasets.Value("string")]})
        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(self.label_classes)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        train_tsv_path = Path(dl_manager.download_and_extract(_URLs["train"]))
        validation_tsv_path = Path(dl_manager.download_and_extract(_URLs["validation"]))
        test_tsv_path = Path(dl_manager.download_and_extract(_URLs["test"]))
        data_files = {
            "train": train_tsv_path,
            "validation": validation_tsv_path,
            "test": test_tsv_path,
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_files["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": data_files["validation"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_files["test"]},
            ),
        ]

    def _generate_examples(self, filepath: Path):
        conll_dataset = load_conll_data(filepath)  # [{'sentence': [T1, T2, ..., Tn], 'labels': [L1, L2, ..., Ln]}]

        if self.config.schema == "source":
            for i, row in enumerate(conll_dataset):
                ex = {"index": str(i), "tokens": row["sentence"], "pos_tag": row["label"]}
                yield i, ex
        elif self.config.schema == "seacrowd_seq_label":
            for i, row in enumerate(conll_dataset):
                ex = {"id": str(i), "tokens": row["sentence"], "labels": row["label"]}
                yield i, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A parallel corpus of KDE4 localization files. The corpus is available in 92 languages in total, with 4099 bitexts.
"""
import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{tiedemann2012parallel,
  title={Parallel Data, Tools and Interfaces in OPUS},
  author={Tiedemann, J{\"o}rg},
  booktitle={Proceedings of the Eighth International Conference on Language Resources and Evaluation (LREC'12)},
  pages={2214--2218},
  year={2012}
}
"""

_DATASETNAME = "kde4"

_DESCRIPTION = """\
A parallel corpus of KDE4 localization files. The corpus is available in 92 languages in total, with 4099 bitexts.
"""

_HOMEPAGE = "https://opus.nlpl.eu/KDE4/corpus/version/KDE4"

_LANGUAGES = ["ind", "khm", "zlm", "tha", "vie"]

_LICENSE = Licenses.UNKNOWN.value
_LOCAL = False

_URL_TEMPLATE = "https://object.pouta.csc.fi/OPUS-KDE4/v2/moses/{src}-{tgt}.txt.zip"

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]
_SOURCE_VERSION = "2.0.0"
_SEACROWD_VERSION = "1.0.0"

kde4_language_codes = {
    "aar": "aa",
    "abk": "ab",
    "ave": "ae",
    "afr": "af",
    "aka": "ak",
    "amh": "am",
    "arg": "an",
    "ara": "ar",
    "asm": "as",
    "ava": "av",
    "aym": "ay",
    "aze": "az",
    "bak": "ba",
    "bel": "be",
    "bul": "bg",
    "bis": "bi",
    "bam": "bm",
    "ben": "bn",
    "bod": "bo",
    "bre": "br",
    "bos": "bs",
    "cat": "ca",
    "che": "ce",
    "cha": "ch",
    "cos": "co",
    "cre": "cr",
    "ces": "cs",
    "chu": "cu",
    "chv": "cv",
    "cym": "cy",
    "dan": "da",
    "deu": "de",
    "div": "dv",
    "dzo": "dz",
    "ewe": "ee",
    "ell": "el",
    "eng": "en",
    "epo": "eo",
    "spa": "es",
    "est": "et",
    "eus": "eu",
    "fas": "fa",
    "ful": "ff",
    "fin": "fi",
    "fij": "fj",
    "fao": "fo",
    "fra": "fr",
    "fry": "fy",
    "gle": "ga",
    "gla": "gd",
    "glg": "gl",
    "grn": "gn",
    "guj": "gu",
    "glv": "gv",
    "hau": "ha",
    "heb": "he",
    "hin": "hi",
    "hmo": "ho",
    "hrv": "hr",
    "hat": "ht",
    "hun": "hu",
    "hye": "hy",
    "her": "hz",
    "ina": "ia",
    "ind": "id",
    "ile": "ie",
    "ibo": "ig",
    "iii": "ii",
    "ipk": "ik",
    "ido": "io",
    "isl": "is",
    "ita": "it",
    "iku": "iu",
    "jpn": "ja",
    "jav": "jv",
    "kat": "ka",
    "kon": "kg",
    "kik": "ki",
    "kua": "kj",
    "kaz": "kk",
    "kal": "kl",
    "khm": "km",
    "kan": "kn",
    "kor": "ko",
    "kau": "kr",
    "kas": "ks",
    "kur": "ku",
    "kom": "kv",
    "cor": "kw",
    "kir": "ky",
    "lat": "la",
    "ltz": "lb",
    "lug": "lg",
    "lim": "li",
    "lin": "ln",
    "lao": "lo",
    "lit": "lt",
    "lub": "lu",
    "lav": "lv",
    "mlg": "mg",
    "mah": "mh",
    "mri": "mi",
    "mkd": "mk",
    "mal": "ml",
    "mon": "mn",
    "mar": "mr",
    "msa": "ms",
    "mlt": "mt",
    "mya": "my",
    "nau": "na",
    "nob": "nb",
    "nde": "nd",
    "nep": "ne",
    "ndo": "ng",
    "nld": "nl",
    "nno": "nn",
    "nor": "no",
    "nbl": "nr",
    "nav": "nv",
    "nya": "ny",
    "oci": "oc",
    "oji": "oj",
    "orm": "om",
    "ori": "or",
    "oss": "os",
    "pan": "pa",
    "pli": "pi",
    "pol": "pl",
    "pus": "ps",
    "por": "pt",
    "que": "qu",
    "roh": "rm",
    "run": "rn",
    "ron": "ro",
    "rus": "ru",
    "kin": "rw",
    "san": "sa",
    "srd": "sc",
    "snd": "sd",
    "sme": "se",
    "sag": "sg",
    "hbs": "sh",
    "sin": "si",
    "slk": "sk",
    "slv": "sl",
    "smo": "sm",
    "sna": "sn",
    "som": "so",
    "sqi": "sq",
    "srp": "sr",
    "ssw": "ss",
    "sot": "st",
    "sun": "su",
    "swe": "sv",
    "swa": "sw",
    "tam": "ta",
    "tel": "te",
    "tgk": "tg",
    "tha": "th",
    "tir": "ti",
    "tuk": "tk",
    "tgl": "tl",
    "tsn": "tn",
    "ton": "to",
    "tur": "tr",
    "tso": "ts",
    "tat": "tt",
    "twi": "tw",
    "tah": "ty",
    "uig": "ug",
    "ukr": "uk",
    "urd": "ur",
    "uzb": "uz",
    "ven": "ve",
    "vie": "vi",
    "vol": "vo",
    "wln": "wa",
    "wol": "wo",
    "xho": "xh",
    "yid": "yi",
    "yor": "yo",
    "zha": "za",
    "zho": "zh",
    "zul": "zu",
    "nds": "nds",
    "mai": "mai",
    "nso": "nso",
    "ast": "ast",
    "crh": "crh",
    "csb": "csb",
    "hne": "hne",
    "hsb": "hsb",
}

configs = {
    "afr": ["msa", "tha", "khm", "ind", "vie"],
    "ara": ["msa", "tha", "khm", "ind", "vie"],
    "asm": ["msa", "tha", "khm", "ind", "vie"],
    "ast": ["msa", "tha", "khm", "ind", "vie"],
    "bel": ["msa", "tha", "khm", "ind", "vie"],
    "bul": ["msa", "tha", "khm", "ind", "vie"],
    "ben": ["msa", "tha", "khm", "ind", "vie"],
    "bre": ["msa", "tha", "khm", "ind", "vie"],
    "cat": ["msa", "tha", "khm", "ind", "vie"],
    "crh": ["msa", "tha", "khm", "ind", "vie"],
    "ces": ["msa", "tha", "khm", "ind", "vie"],
    "csb": ["msa", "tha", "khm", "ind", "vie"],
    "cym": ["msa", "tha", "khm", "ind", "vie"],
    "dan": ["msa", "tha", "khm", "ind", "vie"],
    "deu": ["msa", "tha", "khm", "ind", "vie"],
    "ell": ["msa", "tha", "khm", "ind", "vie"],
    "eng": ["msa", "tha", "khm", "ind", "vie"],
    "epo": ["msa", "tha", "khm", "ind", "vie"],
    "spa": ["msa", "tha", "khm", "ind", "vie"],
    "est": ["msa", "tha", "khm", "ind", "vie"],
    "eus": ["msa", "tha", "khm", "ind", "vie"],
    "fas": ["msa", "tha", "khm", "ind", "vie"],
    "fin": ["msa", "tha", "khm", "ind", "vie"],
    "fra": ["msa", "tha", "khm", "ind", "vie"],
    "fry": ["msa", "tha", "khm", "ind", "vie"],
    "gle": ["msa", "tha", "khm", "ind", "vie"],
    "glg": ["msa", "tha", "khm", "ind", "vie"],
    "guj": ["msa", "tha", "khm", "ind", "vie"],
    "hau": ["msa", "tha", "khm", "ind", "vie"],
    "heb": ["msa", "tha", "khm", "ind", "vie"],
    "hin": ["msa", "tha", "khm", "ind", "vie"],
    "hne": ["msa", "tha", "khm", "ind", "vie"],
    "hrv": ["msa", "tha", "khm", "ind", "vie"],
    "hsb": ["msa", "tha", "khm", "ind", "vie"],
    "hun": ["msa", "tha", "khm", "ind", "vie"],
    "hye": ["msa", "tha", "khm", "ind", "vie"],
    "ind": [
        "kan",
        "pus",
        "msa",
        "slv",
        "tur",
        "rus",
        "nld",
        "mkd",
        "jpn",
        "ori",
        "nep",
        "xho",
        "nds",
        "lav",
        "ukr",
        "vie",
        "mai",
        "tam",
        "ltz",
        "isl",
        "uzb",
        "sme",
        "lit",
        "tgk",
        "kat",
        "mal",
        "srp",
        "wln",
        "por",
        "oci",
        "kur",
        "mar",
        "sin",
        "slk",
        "kor",
        "kaz",
        "ron",
        "nno",
        "tha",
        "khm",
        "tel",
        "ita",
        "pol",
        "swe",
        "pan",
        "nob",
    ],
    "isl": ["khm", "msa", "tha", "vie"],
    "ita": ["khm", "msa", "tha", "vie"],
    "jpn": ["khm", "msa", "tha", "vie"],
    "kat": ["khm", "msa", "tha", "vie"],
    "kaz": ["khm", "msa", "tha", "vie"],
    "khm": [
        "kan",
        "pus",
        "msa",
        "slv",
        "tur",
        "rus",
        "kin",
        "nld",
        "mkd",
        "ori",
        "xho",
        "nso",
        "nep",
        "nds",
        "lav",
        "ukr",
        "vie",
        "mai",
        "tam",
        "ltz",
        "uzb",
        "sme",
        "lit",
        "tgk",
        "mlt",
        "mal",
        "srp",
        "wln",
        "por",
        "oci",
        "kur",
        "mar",
        "sin",
        "slk",
        "kor",
        "ron",
        "nno",
        "tha",
        "tel",
        "pol",
        "swe",
        "pan",
        "nob",
    ],
    "kan": ["msa", "tha", "vie"],
    "kor": ["msa", "tha", "vie"],
    "kur": ["msa", "tha", "vie"],
    "ltz": ["msa", "tha", "vie"],
    "lit": ["msa", "tha", "vie"],
    "lav": ["msa", "tha", "vie"],
    "mai": ["msa", "tha", "vie"],
    "mkd": ["msa", "tha", "vie"],
    "mal": ["msa", "tha", "vie"],
    "mar": ["msa", "tha", "vie"],
    "msa": [
        "pus",
        "slv",
        "tur",
        "rus",
        "kin",
        "nld",
        "ori",
        "xho",
        "nso",
        "nep",
        "nds",
        "ukr",
        "vie",
        "tam",
        "uzb",
        "sme",
        "tgk",
        "mlt",
        "srp",
        "wln",
        "por",
        "oci",
        "slk",
        "sin",
        "ron",
        "nno",
        "tha",
        "tel",
        "pol",
        "swe",
        "pan",
        "nob",
    ],
    "mlt": ["tha"],
    "nob": ["tha", "vie"],
    "nds": ["tha", "vie"],
    "nep": ["tha", "vie"],
    "nld": ["tha", "vie"],
    "nno": ["tha", "vie"],
    "oci": ["tha", "vie"],
    "ori": ["tha", "vie"],
    "pan": ["tha", "vie"],
    "pol": ["tha", "vie"],
    "pus": ["tha", "vie"],
    "por": ["tha", "vie"],
    "ron": ["tha", "vie"],
    "rus": ["tha", "vie"],
    "kin": ["tha"],
    "sme": ["tha", "vie"],
    "sin": ["tha", "vie"],
    "slk": ["tha", "vie"],
    "slv": ["tha", "vie"],
    "srp": ["tha", "vie"],
    "swe": ["tha", "vie"],
    "tam": ["tha", "vie"],
    "tel": ["tha", "vie"],
    "tgk": ["tha", "vie"],
    "tha": ["wln", "tur", "uzb", "xho", "vie", "ukr"],
    "tur": ["vie"],
    "ukr": ["vie"],
    "uzb": ["vie"],
    "vie": ["wln", "xho"],
}


class KDE4Dataset(datasets.GeneratorBasedBuilder):
    """A parallel corpus of KDE4 localization files. The corpus is available in 92 languages in total, with 4099 bitexts."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = (
        [
            SEACrowdConfig(
                name="kde4_source",
                version=datasets.Version(_SOURCE_VERSION),
                description="kde4 source schema for afr to msa",
                schema="source",
                subset_id="afr_msa",
            ),
            SEACrowdConfig(
                name="kde4_seacrowd_t2t",
                version=datasets.Version(_SOURCE_VERSION),
                description="kde4 seacrowd_t2t schema for afr to msa",
                schema="seacrowd_t2t",
                subset_id="afr_msa",
            ),
        ]
        + [
            SEACrowdConfig(
                name=f"kde4_{src_lang}_{tgt_lang}_source",
                version=datasets.Version(_SOURCE_VERSION),
                description=f"kde4 source schema for {src_lang} to {tgt_lang}",
                schema="source",
                subset_id=f"{src_lang}_{tgt_lang}",
            )
            for src_lang in configs
            for tgt_lang in configs[src_lang]
        ]
        + [
            SEACrowdConfig(
                name=f"kde4_{src_lang}_{tgt_lang}_seacrowd_t2t",
                version=datasets.Version(_SEACROWD_VERSION),
                description=f"kde4 seacrowd_t2t schema for {src_lang} to {tgt_lang}",
                schema="seacrowd_t2t",
                subset_id=f"{src_lang}_{tgt_lang}",
            )
            for src_lang in configs
            for tgt_lang in configs[src_lang]
        ]
    )

    DEFAULT_CONFIG_NAME = "kde4_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "src_text": datasets.Value("string"),
                    "tgt_text": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        src_lang, tgt_lang = self.config.subset_id.split("_")
        kde4_src_lang, kde4_tgt_lang = kde4_language_codes[src_lang], kde4_language_codes[tgt_lang]

        url = _URL_TEMPLATE.format(src=kde4_src_lang, tgt=kde4_tgt_lang)
        data_dir = dl_manager.download_and_extract(url)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "src_lang": src_lang,
                    "tgt_lang": tgt_lang,
                    "src_filepath": os.path.join(data_dir, f"KDE4.{kde4_src_lang}-{kde4_tgt_lang}.{kde4_src_lang}"),
                    "tgt_filepath": os.path.join(data_dir, f"KDE4.{kde4_src_lang}-{kde4_tgt_lang}.{kde4_tgt_lang}"),
                },
            )
        ]

    def _generate_examples(self, src_lang: str, tgt_lang: str, src_filepath: Path, tgt_filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        if self.config.schema == "source":
            for row_id, (src_text, tgt_text) in enumerate(zip(open(src_filepath), open(tgt_filepath))):
                yield row_id, {
                    "id": row_id,
                    "src_text": src_text.strip(),
                    "tgt_text": tgt_text.strip(),
                }

        elif self.config.schema == "seacrowd_t2t":
            for row_id, (src_text, tgt_text) in enumerate(zip(open(src_filepath), open(tgt_filepath))):
                yield row_id, {
                    "id": row_id,
                    "text_1": src_text.strip(),
                    "text_2": tgt_text.strip(),
                    "text_1_name": src_lang,
                    "text_2_name": tgt_lang,
                }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
from typing import Dict, List, Tuple
import datasets
import pandas

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks
_CITATION = """
@InProceedings{10.1007/978-3-030-79457-6_35,
author="Luu, Son T.
and Nguyen, Kiet Van
and Nguyen, Ngan Luu-Thuy",
editor="Fujita, Hamido
and Selamat, Ali
and Lin, Jerry Chun-Wei
and Ali, Moonis",
title="A Large-Scale Dataset for Hate Speech Detection on Vietnamese Social Media Texts",
booktitle="Advances and Trends in Artificial Intelligence. Artificial Intelligence Practices",
year="2021",
publisher="Springer International Publishing",
address="Cham",
pages="415--426",
abstract="In recent years, Vietnam witnesses the mass development of social network users on different social
platforms such as Facebook, Youtube, Instagram, and Tiktok. On social media, hate speech has become a critical
problem for social network users. To solve this problem, we introduce the ViHSD - a human-annotated dataset for
automatically detecting hate speech on the social network. This dataset contains over 30,000 comments, each comment
in the dataset has one of three labels: CLEAN, OFFENSIVE, or HATE. Besides, we introduce the data creation process
for annotating and evaluating the quality of the dataset. Finally, we evaluate the dataset by deep learning and transformer models.",
isbn="978-3-030-79457-6"
}
"""

_LOCAL = False
_LANGUAGES = ["vie"]
_DATASETNAME = "uit_vihsd"
_DESCRIPTION = """
The ViHSD dataset consists of comments collected from Facebook pages and YouTube channels that have a
high-interactive rate, and do not restrict comments. This dataset is used for hate speech detection on
Vietnamese language. Data is anonymized, and labeled as either HATE, OFFENSIVE, or CLEAN.
"""

_HOMEPAGE = "https://github.com/sonlam1102/vihsd/"
_LICENSE = Licenses.UNKNOWN.value
_URL = "https://raw.githubusercontent.com/sonlam1102/vihsd/main/data/vihsd.zip"

_Split_Path = {
    "train": "vihsd/train.csv",
    "validation": "vihsd/dev.csv",
    "test": "vihsd/test.csv",
}

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class UiTVihsdDataset(datasets.GeneratorBasedBuilder):
    """
    The SeaCrowd dataloader for the dataset Vietnamese Hate Speech Detection (UIT-ViHSD).
    """

    CLASS_LABELS = ["CLEAN", "OFFENSIVE", "HATE"]  # 0:CLEAN, 1:OFFENSIVE, 2:HATE
    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_text",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema ",
            schema="seacrowd_text",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("int64"),
                    "text": datasets.Value("string"),
                    "label": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(label_names=self.CLASS_LABELS)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        file_paths = dl_manager.download_and_extract(_URL)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": os.path.join(file_paths, _Split_Path["train"])},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": os.path.join(file_paths, _Split_Path["validation"])},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(file_paths, _Split_Path["test"])},
            ),
        ]

    def _generate_examples(self, filepath) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        data_lines = pandas.read_csv(filepath)
        for row in data_lines.itertuples():
            if self.config.schema == "source":
                example = {"id": str(row.Index), "text": row.free_text, "label": row.label_id}
            if self.config.schema == "seacrowd_text":
                example = {"id": str(row.Index), "text": row.free_text, "label": self.CLASS_LABELS[int(row.label_id)]}
            yield row.Index, example




import json
from pathlib import Path
from typing import List

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import DEFAULT_SEACROWD_VIEW_NAME, DEFAULT_SOURCE_VIEW_NAME, Licenses, Tasks

_DATASETNAME = "thai_depression"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["tha"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False
_CITATION = """\
@inproceedings{hamalainen-etal-2021-detecting,
    title = "Detecting Depression in Thai Blog Posts: a Dataset and a Baseline",
    author = {H{\"a}m{\"a}l{\"a}inen, Mika  and
      Patpong, Pattama  and
      Alnajjar, Khalid  and
      Partanen, Niko  and
      Rueter, Jack},
    editor = "Xu, Wei  and
      Ritter, Alan  and
      Baldwin, Tim  and
      Rahimi, Afshin",
    booktitle = "Proceedings of the Seventh Workshop on Noisy User-generated Text (W-NUT 2021)",
    month = nov,
    year = "2021",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2021.wnut-1.3",
    doi = "10.18653/v1/2021.wnut-1.3",
    pages = "20--25",
    abstract = "We present the first openly available corpus for detecting depression in Thai. Our corpus is compiled by expert verified cases of depression in several online blogs.
    We experiment with two different LSTM based models and two different BERT based models. We achieve a 77.53%% accuracy with a Thai BERT model in detecting depression.
    This establishes a good baseline for future researcher on the same corpus. Furthermore, we identify a need for Thai embeddings that have been trained on a more varied corpus than Wikipedia.
    Our corpus, code and trained models have been released openly on Zenodo.",
}
"""

_DESCRIPTION = """\
We present the first openly available corpus for detecting depression in Thai. Our corpus is compiled by expert verified cases of depression in several online blogs.
We experiment with two different LSTM based models and two different BERT based models. We achieve a 77.53%% accuracy with a Thai BERT model in detecting depression.
This establishes a good baseline for future researcher on the same corpus. Furthermore, we identify a need for Thai embeddings that have been trained on a more varied corpus than Wikipedia.
Our corpus, code and trained models have been released openly on Zenodo.
"""

_HOMEPAGE = "https://zenodo.org/records/4734552"

_LICENSE = Licenses.CC_BY_NC_ND_4_0.value

_URLs = "https://zenodo.org/records/4734552/files/data.zip?download=1"

_SUPPORTED_TASKS = [Tasks.EMOTION_CLASSIFICATION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class ThaiDepressionDataset(datasets.GeneratorBasedBuilder):
    """Thai depression detection dataset."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_text",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} seacrowd schema",
            schema="seacrowd_text",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "label": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(["depression", "no_depression"])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        path = Path(dl_manager.download_and_extract(_URLs))
        data_files = {
            "train": path / "splits/train.json",
            "test": path / "splits/test.json",
            "valid": path / "splits/valid.json",
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_files["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": data_files["valid"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_files["test"]},
            ),
        ]

    def _parse_and_label(self, file_path):
        with open(file_path, "r", encoding="utf-8") as file:
            data = json.load(file)

        parsed_data = []
        for item in data:
            parsed_data.append({"text": item[0], "label": item[1]})

        return parsed_data

    def _generate_examples(self, filepath: Path):
        print("Reading ", filepath)
        for id, row in enumerate(self._parse_and_label(filepath)):
            if self.config.schema == "source":
                yield id, {"text": row["text"], "label": row["label"]}
            elif self.config.schema == "seacrowd_text":
                yield id, {"id": str(id), "text": row["text"], "label": row["label"]}
            else:
                raise ValueError(f"Invalid config: {self.config.name}")



"""
Utility for filtering and loading SEACrowd datasets.
"""
from collections import Counter
from importlib.machinery import SourceFileLoader
import logging
import os
import pathlib
from tqdm import tqdm
from types import ModuleType
from typing import Callable, Iterable, List, Optional, Dict

from dataclasses import dataclass
from dataclasses import field
import datasets

from .utils.configs import SEACrowdConfig
from .utils.constants import Tasks, SCHEMA_TO_TASKS
import pandas as pd

_LARGE_CONFIG_NAMES = [
    'covost2_ind_eng_seacrowd_sptext',
    'covost2_eng_ind_seacrowd_sptext',
    'covost2_ind_eng_seacrowd_t2t',
    'covost2_eng_ind_seacrowd_t2t',
    'cc100_ind_source',
    'cc100_jav_source',
    'cc100_sun_source',
    'cc100_ind_seacrowd_ssp',
    'cc100_jav_seacrowd_ssp',
    'cc100_sun_seacrowd_ssp',
    'indo4b_source', 'indo4b_seacrowd_ssp',
    'indo4b_plus_source', 'indo4b_plus_seacrowd_ssp',
    'kopi_cc_all-raw_source',
    'kopi_cc_all-dedup_source',
    'kopi_cc_all-neardup_source',
    'kopi_cc_all-neardup_clean_source',
    'kopi_cc_2021_10-dedup_source',
    'kopi_cc_2021_10-neardup_source',
    'kopi_cc_2021_10-neardup_clean_source',
    'kopi_cc_2021_17-raw_source',
    'kopi_cc_2021_17-dedup_source',
    'kopi_cc_2021_17-neardup_source',
    'kopi_cc_2021_17-neardup_clean_source',
    'kopi_cc_2021_21-raw_source',
    'kopi_cc_2021_21-dedup_source',
    'kopi_cc_2021_21-neardup_source',
    'kopi_cc_2021_21-neardup_clean_source',
    'kopi_cc_2021_25-raw_source',
    'kopi_cc_2021_25-dedup_source',
    'kopi_cc_2021_25-neardup_source',
    'kopi_cc_2021_25-neardup_clean_source',
    'kopi_cc_2021_31-raw_source',
    'kopi_cc_2021_31-dedup_source',
    'kopi_cc_2021_31-neardup_source',
    'kopi_cc_2021_31-neardup_clean_source',
    'kopi_cc_2021_39-raw_source',
    'kopi_cc_2021_39-dedup_source',
    'kopi_cc_2021_39-neardup_source',
    'kopi_cc_2021_39-neardup_clean_source',
    'kopi_cc_2021_43-raw_source',
    'kopi_cc_2021_43-dedup_source',
    'kopi_cc_2021_43-neardup_source',
    'kopi_cc_2021_43-neardup_clean_source',
    'kopi_cc_2021_49-dedup_source',
    'kopi_cc_2021_49-neardup_source',
    'kopi_cc_2021_49-neardup_clean_source',
    'kopi_cc_2022_05-raw_source',
    'kopi_cc_2022_05-dedup_source',
    'kopi_cc_2022_05-neardup_source',
    'kopi_cc_2022_05-neardup_clean_source',
    'kopi_cc_2022_21-raw_source',
    'kopi_cc_2022_21-dedup_source',
    'kopi_cc_2022_21-neardup_source',
    'kopi_cc_2022_21-neardup_clean_source',
    'kopi_cc_2022_27-raw_source',
    'kopi_cc_2022_27-dedup_source',
    'kopi_cc_2022_27-neardup_source',
    'kopi_cc_2022_27-neardup_clean_source',
    'kopi_cc_all-raw_seacrowd_ssp',
    'kopi_cc_all-dedup_seacrowd_ssp',
    'kopi_cc_all-neardup_seacrowd_ssp',
    'kopi_cc_all-neardup_clean_seacrowd_ssp',
    'kopi_cc_2021_10-dedup_seacrowd_ssp',
    'kopi_cc_2021_10-neardup_seacrowd_ssp',
    'kopi_cc_2021_10-neardup_clean_seacrowd_ssp',
    'kopi_cc_2021_17-raw_seacrowd_ssp',
    'kopi_cc_2021_17-dedup_seacrowd_ssp',
    'kopi_cc_2021_17-neardup_seacrowd_ssp',
    'kopi_cc_2021_17-neardup_clean_seacrowd_ssp',
    'kopi_cc_2021_21-raw_seacrowd_ssp',
    'kopi_cc_2021_21-dedup_seacrowd_ssp',
    'kopi_cc_2021_21-neardup_seacrowd_ssp',
    'kopi_cc_2021_21-neardup_clean_seacrowd_ssp',
    'kopi_cc_2021_25-raw_seacrowd_ssp',
    'kopi_cc_2021_25-dedup_seacrowd_ssp',
    'kopi_cc_2021_25-neardup_seacrowd_ssp',
    'kopi_cc_2021_25-neardup_clean_seacrowd_ssp',
    'kopi_cc_2021_31-raw_seacrowd_ssp',
    'kopi_cc_2021_31-dedup_seacrowd_ssp',
    'kopi_cc_2021_31-neardup_seacrowd_ssp',
    'kopi_cc_2021_31-neardup_clean_seacrowd_ssp',
    'kopi_cc_2021_39-raw_seacrowd_ssp',
    'kopi_cc_2021_39-dedup_seacrowd_ssp',
    'kopi_cc_2021_39-neardup_seacrowd_ssp',
    'kopi_cc_2021_39-neardup_clean_seacrowd_ssp',
    'kopi_cc_2021_43-raw_seacrowd_ssp',
    'kopi_cc_2021_43-dedup_seacrowd_ssp',
    'kopi_cc_2021_43-neardup_seacrowd_ssp',
    'kopi_cc_2021_43-neardup_clean_seacrowd_ssp',
    'kopi_cc_2021_49-dedup_seacrowd_ssp',
    'kopi_cc_2021_49-neardup_seacrowd_ssp',
    'kopi_cc_2021_49-neardup_clean_seacrowd_ssp',
    'kopi_cc_2022_05-raw_seacrowd_ssp',
    'kopi_cc_2022_05-dedup_seacrowd_ssp',
    'kopi_cc_2022_05-neardup_seacrowd_ssp',
    'kopi_cc_2022_05-neardup_clean_seacrowd_ssp',
    'kopi_cc_2022_21-raw_seacrowd_ssp',
    'kopi_cc_2022_21-dedup_seacrowd_ssp',
    'kopi_cc_2022_21-neardup_seacrowd_ssp',
    'kopi_cc_2022_21-neardup_clean_seacrowd_ssp',
    'kopi_cc_2022_27-raw_seacrowd_ssp',
    'kopi_cc_2022_27-dedup_seacrowd_ssp',
    'kopi_cc_2022_27-neardup_seacrowd_ssp',
    'kopi_cc_2022_27-neardup_clean_seacrowd_ssp',
    'kopi_cc_news_2016_source',
    'kopi_cc_news_2017_source',
    'kopi_cc_news_2018_source',
    'kopi_cc_news_2019_source',
    'kopi_cc_news_2020_source',
    'kopi_cc_news_2021_source',
    'kopi_cc_news_2022_source',
    'kopi_cc_news_all_source',
    'kopi_cc_news_2016_seacrowd_ssp',
    'kopi_cc_news_2017_seacrowd_ssp',
    'kopi_cc_news_2018_seacrowd_ssp',
    'kopi_cc_news_2019_seacrowd_ssp',
    'kopi_cc_news_2020_seacrowd_ssp',
    'kopi_cc_news_2021_seacrowd_ssp',
    'kopi_cc_news_2022_seacrowd_ssp',
    'kopi_cc_news_all_seacrowd_ssp',
    'kopi_nllb_all-raw_source',
    'kopi_nllb_all-dedup_source',
    'kopi_nllb_all-neardup_source',
    'kopi_nllb_ace_Latn-raw_source',
    'kopi_nllb_ace_Latn-dedup_source',
    'kopi_nllb_ace_Latn-neardup_source',
    'kopi_nllb_ban_Latn-raw_source',
    'kopi_nllb_ban_Latn-dedup_source',
    'kopi_nllb_ban_Latn-neardup_source',
    'kopi_nllb_bjn_Latn-raw_source',
    'kopi_nllb_bjn_Latn-dedup_source',
    'kopi_nllb_bjn_Latn-neardup_source',
    'kopi_nllb_ind_Latn-raw_source',
    'kopi_nllb_ind_Latn-dedup_source',
    'kopi_nllb_ind_Latn-neardup_source',
    'kopi_nllb_jav_Latn-raw_source',
    'kopi_nllb_jav_Latn-dedup_source',
    'kopi_nllb_jav_Latn-neardup_source',
    'kopi_nllb_min_Latn-raw_source',
    'kopi_nllb_min_Latn-dedup_source',
    'kopi_nllb_min_Latn-neardup_source',
    'kopi_nllb_sun_Latn-raw_source',
    'kopi_nllb_sun_Latn-dedup_source',
    'kopi_nllb_sun_Latn-neardup_source',
    'kopi_nllb_all-raw_seacrowd_ssp',
    'kopi_nllb_all-dedup_seacrowd_ssp',
    'kopi_nllb_all-neardup_seacrowd_ssp',
    'kopi_nllb_ace_Latn-raw_seacrowd_ssp',
    'kopi_nllb_ace_Latn-dedup_seacrowd_ssp',
    'kopi_nllb_ace_Latn-neardup_seacrowd_ssp',
    'kopi_nllb_ban_Latn-raw_seacrowd_ssp',
    'kopi_nllb_ban_Latn-dedup_seacrowd_ssp',
    'kopi_nllb_ban_Latn-neardup_seacrowd_ssp',
    'kopi_nllb_bjn_Latn-raw_seacrowd_ssp',
    'kopi_nllb_bjn_Latn-dedup_seacrowd_ssp',
    'kopi_nllb_bjn_Latn-neardup_seacrowd_ssp',
    'kopi_nllb_ind_Latn-raw_seacrowd_ssp',
    'kopi_nllb_ind_Latn-dedup_seacrowd_ssp',
    'kopi_nllb_ind_Latn-neardup_seacrowd_ssp',
    'kopi_nllb_jav_Latn-raw_seacrowd_ssp',
    'kopi_nllb_jav_Latn-dedup_seacrowd_ssp',
    'kopi_nllb_jav_Latn-neardup_seacrowd_ssp',
    'kopi_nllb_min_Latn-raw_seacrowd_ssp',
    'kopi_nllb_min_Latn-dedup_seacrowd_ssp',
    'kopi_nllb_min_Latn-neardup_seacrowd_ssp',
    'kopi_nllb_sun_Latn-raw_seacrowd_ssp',
    'kopi_nllb_sun_Latn-dedup_seacrowd_ssp',
    'kopi_nllb_sun_Latn-neardup_seacrowd_ssp'
]

_RESOURCE_CONFIG_NAMES = [
    'inset_lexicon_seacrowd_text',
    'kamus_alay_seacrowd_t2t'
]

_CURRENTLY_BROKEN_NAMES = [

]

BENCHMARK_DICT = {
    'IndoNLU': [
        'emot_seacrowd_text',
        'smsa_seacrowd_text',
        'wrete_seacrowd_pairs',
        'casa_seacrowd_text_multi',
        'hoasa_seacrowd_text_multi',
        'facqa_seacrowd_qa',
        'indonlu_nergrit_seacrowd_seq_label',
        'nerp_seacrowd_seq_label',
        'posp_seacrowd_seq_label',
        'term_a_seacrowd_seq_label',
        'keps_seacrowd_seq_label',
        'idn_tagged_corpus_csui_seacrowd_seq_label'
    ],
    'IndoNLG': [
        # MT
        'bible_en_id_seacrowd_t2t',
        'bible_su_id_seacrowd_t2t',
        'bible_jv_id_seacrowd_t2t',
        'ted_en_id_seacrowd_t2t',
        'indo_general_mt_en_id_seacrowd_t2t',
        'news_en_id_seacrowd_t2t',        
        # Summarization
        'indosum_fold0_seacrowd_t2t',
        'liputan6_canonical_seacrowd_t2t',
        'liputan6_xtreme_seacrowd_t2t',
        # Chit Chat
        'xpersona_id_seacrowd_t2t',
        # QA
        'tydiqa_id_seacrowd_qa',
    ],
    'IndoLEM': [
        'indolem_ntp_seacrowd_pairs',
        'indolem_sentiment_seacrowd_text',
        'indolem_ner_ugm_fold0_seacrowd_seq_label',
        'indolem_ner_ugm_fold1_seacrowd_seq_label',
        'indolem_ner_ugm_fold2_seacrowd_seq_label',
        'indolem_ner_ugm_fold3_seacrowd_seq_label',
        'indolem_ner_ugm_fold4_seacrowd_seq_label',
        'indolem_ud_id_gsd_seacrowd_kb',
        'indolem_ud_id_pud_seacrowd_kb',
        'indolem_tweet_ordering_seacrowd_seq_label',
        'indolem_nerui_fold0_seacrowd_seq_label',
        'indolem_nerui_fold1_seacrowd_seq_label',
        'indolem_nerui_fold2_seacrowd_seq_label',
        'indolem_nerui_fold3_seacrowd_seq_label',
        'indolem_nerui_fold4_seacrowd_seq_label'
    ],
    'NusaX': [
        # Ind - XXX
        'nusax_mt_ind_ace_seacrowd_t2t',
        'nusax_mt_ind_ban_seacrowd_t2t',
        'nusax_mt_ind_bjn_seacrowd_t2t',
        'nusax_mt_ind_bug_seacrowd_t2t',
        'nusax_mt_ind_eng_seacrowd_t2t',
        'nusax_mt_ind_jav_seacrowd_t2t',
        'nusax_mt_ind_mad_seacrowd_t2t',
        'nusax_mt_ind_min_seacrowd_t2t',
        'nusax_mt_ind_nij_seacrowd_t2t',
        'nusax_mt_ind_sun_seacrowd_t2t',
        'nusax_mt_ind_bbc_seacrowd_t2t',
        
        # XXX - Ind
        'nusax_mt_ace_ind_seacrowd_t2t',
        'nusax_mt_ban_ind_seacrowd_t2t',
        'nusax_mt_bjn_ind_seacrowd_t2t',
        'nusax_mt_bug_ind_seacrowd_t2t',
        'nusax_mt_eng_ind_seacrowd_t2t',
        'nusax_mt_jav_ind_seacrowd_t2t',
        'nusax_mt_mad_ind_seacrowd_t2t',
        'nusax_mt_min_ind_seacrowd_t2t',
        'nusax_mt_nij_ind_seacrowd_t2t',
        'nusax_mt_sun_ind_seacrowd_t2t',
        'nusax_mt_bbc_ind_seacrowd_t2t',    
    ],    
    'NusaNLU': [
        'emot_seacrowd_text',
        'emotcmt_seacrowd_text',
        'emotion_id_opinion_seacrowd_text',
        'id_abusive_seacrowd_text',
        'id_google_play_review_seacrowd_text',
        'id_google_play_review_posneg_seacrowd_text',
        'id_hatespeech_seacrowd_text',
        'imdb_jv_seacrowd_text',
        'indolem_sentiment_seacrowd_text',
        'jadi_ide_seacrowd_text',
        'nusax_senti_ace_seacrowd_text',
        'nusax_senti_ban_seacrowd_text',
        'nusax_senti_bjn_seacrowd_text',
        'nusax_senti_bug_seacrowd_text',
        'nusax_senti_eng_seacrowd_text',
        'nusax_senti_ind_seacrowd_text',
        'nusax_senti_jav_seacrowd_text',
        'nusax_senti_mad_seacrowd_text',
        'nusax_senti_min_seacrowd_text',
        'nusax_senti_nij_seacrowd_text',
        'nusax_senti_sun_seacrowd_text',
        'nusax_senti_bbc_seacrowd_text',
        'sentiment_nathasa_review_seacrowd_text',
        'smsa_seacrowd_text',    
        'indolem_ntp_seacrowd_pairs',
        'indonli_seacrowd_pairs',
        'code_mixed_jv_id_jv_seacrowd_text',
        'code_mixed_jv_id_id_seacrowd_text',
        'id_am2ico_seacrowd_pairs',
        'id_abusive_news_comment_seacrowd_text',
        'id_hoax_news_seacrowd_text',
        'id_hsd_nofaaulia_seacrowd_text',
        'id_stance_seacrowd_pairs',
        'indo_law_seacrowd_text',
        'indotacos_seacrowd_text',
        'karonese_sentiment_seacrowd_text',
        'su_emot_seacrowd_text',
        'wrete_seacrowd_pairs',
        'id_short_answer_grading_seacrowd_pairs'
    ],    
    'NusaNLG': [
        'bible_en_id_seacrowd_t2t',
        'bible_jv_id_seacrowd_t2t',
        'bible_su_id_seacrowd_t2t',
        'id_panl_bppt_seacrowd_t2t',
        'indo_general_mt_en_id_seacrowd_t2t',
        'indo_religious_mt_en_id_seacrowd_t2t',
        'minangnlp_mt_seacrowd_t2t',
        'news_en_id_seacrowd_t2t',
        'nusax_mt_ace_ind_seacrowd_t2t',
        'nusax_mt_ban_ind_seacrowd_t2t',
        'nusax_mt_bjn_ind_seacrowd_t2t',
        'nusax_mt_bug_ind_seacrowd_t2t',
        'nusax_mt_eng_ind_seacrowd_t2t',
        'nusax_mt_ind_ace_seacrowd_t2t',
        'nusax_mt_ind_ban_seacrowd_t2t',
        'nusax_mt_ind_bjn_seacrowd_t2t',
        'nusax_mt_ind_bug_seacrowd_t2t',
        'nusax_mt_ind_eng_seacrowd_t2t',
        'nusax_mt_ind_jav_seacrowd_t2t',
        'nusax_mt_ind_mad_seacrowd_t2t',
        'nusax_mt_ind_min_seacrowd_t2t',
        'nusax_mt_ind_nij_seacrowd_t2t',
        'nusax_mt_ind_sun_seacrowd_t2t',
        'nusax_mt_ind_bbc_seacrowd_t2t',
        'nusax_mt_jav_ind_seacrowd_t2t',
        'nusax_mt_mad_ind_seacrowd_t2t',
        'nusax_mt_min_ind_seacrowd_t2t',
        'nusax_mt_nij_ind_seacrowd_t2t',
        'nusax_mt_sun_ind_seacrowd_t2t',
        'nusax_mt_bbc_ind_seacrowd_t2t',
        'parallel_su_id_seacrowd_t2t',
        'ted_en_id_seacrowd_t2t',
        'ud_id_csui_seacrowd_t2t',
        'korpus_seacrowd_ind_jav_seacrowd_t2t',
        'korpus_seacrowd_ind_xdy_seacrowd_t2t',
        'korpus_seacrowd_ind_bug_seacrowd_t2t',
        'korpus_seacrowd_ind_sun_seacrowd_t2t',
        'korpus_seacrowd_ind_mad_seacrowd_t2t',
        'korpus_seacrowd_ind_bjn_seacrowd_t2t',
        'korpus_seacrowd_ind_bbc_seacrowd_t2t',
        'korpus_seacrowd_ind_khek_seacrowd_t2t',
        'korpus_seacrowd_ind_msa_seacrowd_t2t',
        'korpus_seacrowd_ind_min_seacrowd_t2t',
        'korpus_seacrowd_ind_tiociu_seacrowd_t2t',
        'korpus_seacrowd_jav_ind_seacrowd_t2t',
        'korpus_seacrowd_xdy_ind_seacrowd_t2t',
        'korpus_seacrowd_bug_ind_seacrowd_t2t',
        'korpus_seacrowd_sun_ind_seacrowd_t2t',
        'korpus_seacrowd_mad_ind_seacrowd_t2t',
        'korpus_seacrowd_bjn_ind_seacrowd_t2t',
        'korpus_seacrowd_bbc_ind_seacrowd_t2t',
        'korpus_seacrowd_khek_ind_seacrowd_t2t',
        'korpus_seacrowd_msa_ind_seacrowd_t2t',
        'korpus_seacrowd_min_ind_seacrowd_t2t',
        'korpus_seacrowd_tiociu_ind_seacrowd_t2t',
        'indosum_fold0_seacrowd_t2t',
        'liputan6_canonical_seacrowd_t2t',
        'xl_sum_seacrowd_t2t',
        'id_qqp_seacrowd_t2t',
        'multilexnorm_seacrowd_t2t',
        'paracotta_id_seacrowd_t2t',
        'stif_indonesia_seacrowd_t2t',        
        'xpersona_id_seacrowd_t2t'
        'facqa_seacrowd_qa',
        'idk_mrc_seacrowd_qa',
        'tydiqa_id_seacrowd_qa'
    ],
    'NusaASR': [
        # Ind
        'indspeech_digit_cdsr_seacrowd_sptext',
        'indspeech_news_lvcsr_seacrowd_sptext',
        'indspeech_teldialog_lvcsr_seacrowd_sptext',
        'indspeech_teldialog_svcsr_seacrowd_sptext',
        'librivox_indonesia_ind_seacrowd_sptext',
        'titml_idn_seacrowd_sptext'
        # Sun
        'indspeech_newstra_ethnicsr_nooverlap_sun_seacrowd_sptext',
        'indspeech_news_ethnicsr_su_nooverlap_seacrowd_sptext',
        'librivox_indonesia_sun_seacrowd_sptext',
        'su_id_asr_seacrowd_sptext',
        # Jav
        'indspeech_newstra_ethnicsr_nooverlap_jav_seacrowd_sptext',
        'indspeech_news_ethnicsr_jv_nooverlap_seacrowd_sptext',
        'librivox_indonesia_jav_seacrowd_sptext',
        'jv_id_asr_seacrowd_sptext',
        # Ban
        'indspeech_newstra_ethnicsr_nooverlap_ban_seacrowd_sptext',
        'librivox_indonesia_ban_seacrowd_sptext',
        # Btk
        'indspeech_newstra_ethnicsr_nooverlap_btk_seacrowd_sptext',
        # Ace
        'librivox_indonesia_ace_seacrowd_sptext',
        # Bug
        'librivox_indonesia_bug_seacrowd_sptext',
        # Min
        'librivox_indonesia_min_seacrowd_sptext',
    ],
    'NusaTranslation': [
        # NusaTranslation Senti
        'nusatranslation_senti_abs_seacrowd_text',
        'nusatranslation_senti_btk_seacrowd_text',
        'nusatranslation_senti_bew_seacrowd_text',
        'nusatranslation_senti_bhp_seacrowd_text',
        'nusatranslation_senti_jav_seacrowd_text',
        'nusatranslation_senti_mad_seacrowd_text',
        'nusatranslation_senti_mak_seacrowd_text',
        'nusatranslation_senti_min_seacrowd_text',
        'nusatranslation_senti_mui_seacrowd_text',
        'nusatranslation_senti_rej_seacrowd_text',
        'nusatranslation_senti_sun_seacrowd_text',

        # NusaTranslation Emot
        'nusatranslation_emot_abs_seacrowd_text',
        'nusatranslation_emot_btk_seacrowd_text',
        'nusatranslation_emot_bew_seacrowd_text',
        'nusatranslation_emot_bhp_seacrowd_text',
        'nusatranslation_emot_jav_seacrowd_text',
        'nusatranslation_emot_mad_seacrowd_text',
        'nusatranslation_emot_mak_seacrowd_text',
        'nusatranslation_emot_min_seacrowd_text',
        'nusatranslation_emot_mui_seacrowd_text',
        'nusatranslation_emot_rej_seacrowd_text',
        'nusatranslation_emot_sun_seacrowd_text',

        # NusaTranslation MT Ind-XXX
        'nusatranslation_mt_ind_abs_seacrowd_t2t',
        'nusatranslation_mt_ind_btk_seacrowd_t2t',
        'nusatranslation_mt_ind_bew_seacrowd_t2t',
        'nusatranslation_mt_ind_bhp_seacrowd_t2t',
        'nusatranslation_mt_ind_jav_seacrowd_t2t',
        'nusatranslation_mt_ind_mad_seacrowd_t2t',
        'nusatranslation_mt_ind_mak_seacrowd_t2t',
        'nusatranslation_mt_ind_min_seacrowd_t2t',
        'nusatranslation_mt_ind_mui_seacrowd_t2t',
        'nusatranslation_mt_ind_rej_seacrowd_t2t',
        'nusatranslation_mt_ind_sun_seacrowd_t2t',

        # NusaTranslation MT XXX-Ind
        'nusatranslation_mt_abs_ind_seacrowd_t2t',
        'nusatranslation_mt_btk_ind_seacrowd_t2t',
        'nusatranslation_mt_bew_ind_seacrowd_t2t',
        'nusatranslation_mt_bhp_ind_seacrowd_t2t',
        'nusatranslation_mt_jav_ind_seacrowd_t2t',
        'nusatranslation_mt_mad_ind_seacrowd_t2t',
        'nusatranslation_mt_mak_ind_seacrowd_t2t',
        'nusatranslation_mt_min_ind_seacrowd_t2t',
        'nusatranslation_mt_mui_ind_seacrowd_t2t',
        'nusatranslation_mt_rej_ind_seacrowd_t2t',
        'nusatranslation_mt_sun_ind_seacrowd_t2t',
    ],
    'NusaParagraph': [
        # NusaParagraph Topic
        'nusaparagraph_topic_btk_seacrowd_text',
        'nusaparagraph_topic_bew_seacrowd_text',
        'nusaparagraph_topic_bug_seacrowd_text',
        'nusaparagraph_topic_jav_seacrowd_text',
        'nusaparagraph_topic_mad_seacrowd_text',
        'nusaparagraph_topic_mak_seacrowd_text',
        'nusaparagraph_topic_min_seacrowd_text',
        'nusaparagraph_topic_mui_seacrowd_text',
        'nusaparagraph_topic_rej_seacrowd_text',
        'nusaparagraph_topic_sun_seacrowd_text',

        # NusaParagraph Rhetoric
        'nusaparagraph_rhetoric_btk_seacrowd_text',
        'nusaparagraph_rhetoric_bew_seacrowd_text',
        'nusaparagraph_rhetoric_bug_seacrowd_text',
        'nusaparagraph_rhetoric_jav_seacrowd_text',
        'nusaparagraph_rhetoric_mad_seacrowd_text',
        'nusaparagraph_rhetoric_mak_seacrowd_text',
        'nusaparagraph_rhetoric_min_seacrowd_text',
        'nusaparagraph_rhetoric_mui_seacrowd_text',
        'nusaparagraph_rhetoric_rej_seacrowd_text',
        'nusaparagraph_rhetoric_sun_seacrowd_text',

        # NusaParagraph Emot
        'nusaparagraph_emot_btk_seacrowd_text',
        'nusaparagraph_emot_bew_seacrowd_text',
        'nusaparagraph_emot_bug_seacrowd_text',
        'nusaparagraph_emot_jav_seacrowd_text',
        'nusaparagraph_emot_mad_seacrowd_text',
        'nusaparagraph_emot_mak_seacrowd_text',
        'nusaparagraph_emot_min_seacrowd_text',
        'nusaparagraph_emot_mui_seacrowd_text',
        'nusaparagraph_emot_rej_seacrowd_text',
        'nusaparagraph_emot_sun_seacrowd_text',
    ],
    'NusaWrites': [
        # NusaTranslation Senti
        'nusatranslation_senti_abs_seacrowd_text',
        'nusatranslation_senti_btk_seacrowd_text',
        'nusatranslation_senti_bew_seacrowd_text',
        'nusatranslation_senti_bhp_seacrowd_text',
        'nusatranslation_senti_jav_seacrowd_text',
        'nusatranslation_senti_mad_seacrowd_text',
        'nusatranslation_senti_mak_seacrowd_text',
        'nusatranslation_senti_min_seacrowd_text',
        'nusatranslation_senti_mui_seacrowd_text',
        'nusatranslation_senti_rej_seacrowd_text',
        'nusatranslation_senti_sun_seacrowd_text',

        # NusaTranslation Emot
        'nusatranslation_emot_abs_seacrowd_text',
        'nusatranslation_emot_btk_seacrowd_text',
        'nusatranslation_emot_bew_seacrowd_text',
        'nusatranslation_emot_bhp_seacrowd_text',
        'nusatranslation_emot_jav_seacrowd_text',
        'nusatranslation_emot_mad_seacrowd_text',
        'nusatranslation_emot_mak_seacrowd_text',
        'nusatranslation_emot_min_seacrowd_text',
        'nusatranslation_emot_mui_seacrowd_text',
        'nusatranslation_emot_rej_seacrowd_text',
        'nusatranslation_emot_sun_seacrowd_text',

        # NusaTranslation MT Ind-XXX
        'nusatranslation_mt_ind_abs_seacrowd_t2t',
        'nusatranslation_mt_ind_btk_seacrowd_t2t',
        'nusatranslation_mt_ind_bew_seacrowd_t2t',
        'nusatranslation_mt_ind_bhp_seacrowd_t2t',
        'nusatranslation_mt_ind_jav_seacrowd_t2t',
        'nusatranslation_mt_ind_mad_seacrowd_t2t',
        'nusatranslation_mt_ind_mak_seacrowd_t2t',
        'nusatranslation_mt_ind_min_seacrowd_t2t',
        'nusatranslation_mt_ind_mui_seacrowd_t2t',
        'nusatranslation_mt_ind_rej_seacrowd_t2t',
        'nusatranslation_mt_ind_sun_seacrowd_t2t',

        # NusaTranslation MT XXX-Ind
        'nusatranslation_mt_abs_ind_seacrowd_t2t',
        'nusatranslation_mt_btk_ind_seacrowd_t2t',
        'nusatranslation_mt_bew_ind_seacrowd_t2t',
        'nusatranslation_mt_bhp_ind_seacrowd_t2t',
        'nusatranslation_mt_jav_ind_seacrowd_t2t',
        'nusatranslation_mt_mad_ind_seacrowd_t2t',
        'nusatranslation_mt_mak_ind_seacrowd_t2t',
        'nusatranslation_mt_min_ind_seacrowd_t2t',
        'nusatranslation_mt_mui_ind_seacrowd_t2t',
        'nusatranslation_mt_rej_ind_seacrowd_t2t',
        'nusatranslation_mt_sun_ind_seacrowd_t2t',

        # NusaParagraph Topic
        'nusaparagraph_topic_btk_seacrowd_text',
        'nusaparagraph_topic_bew_seacrowd_text',
        'nusaparagraph_topic_bug_seacrowd_text',
        'nusaparagraph_topic_jav_seacrowd_text',
        'nusaparagraph_topic_mad_seacrowd_text',
        'nusaparagraph_topic_mak_seacrowd_text',
        'nusaparagraph_topic_min_seacrowd_text',
        'nusaparagraph_topic_mui_seacrowd_text',
        'nusaparagraph_topic_rej_seacrowd_text',
        'nusaparagraph_topic_sun_seacrowd_text',

        # NusaParagraph Rhetoric
        'nusaparagraph_rhetoric_btk_seacrowd_text',
        'nusaparagraph_rhetoric_bew_seacrowd_text',
        'nusaparagraph_rhetoric_bug_seacrowd_text',
        'nusaparagraph_rhetoric_jav_seacrowd_text',
        'nusaparagraph_rhetoric_mad_seacrowd_text',
        'nusaparagraph_rhetoric_mak_seacrowd_text',
        'nusaparagraph_rhetoric_min_seacrowd_text',
        'nusaparagraph_rhetoric_mui_seacrowd_text',
        'nusaparagraph_rhetoric_rej_seacrowd_text',
        'nusaparagraph_rhetoric_sun_seacrowd_text',

        # NusaParagraph Emot
        'nusaparagraph_emot_btk_seacrowd_text',
        'nusaparagraph_emot_bew_seacrowd_text',
        'nusaparagraph_emot_bug_seacrowd_text',
        'nusaparagraph_emot_jav_seacrowd_text',
        'nusaparagraph_emot_mad_seacrowd_text',
        'nusaparagraph_emot_mak_seacrowd_text',
        'nusaparagraph_emot_min_seacrowd_text',
        'nusaparagraph_emot_mui_seacrowd_text',
        'nusaparagraph_emot_rej_seacrowd_text',
        'nusaparagraph_emot_sun_seacrowd_text',
    ],
}

@dataclass
class SEACrowdMetadata:
    """Metadata for one config of a dataset."""

    script: pathlib.Path
    dataset_name: str
    tasks: List[Tasks]
    languages: List[str]
    config: SEACrowdConfig
    is_local: bool
    is_seacrowd_schema: bool
    seacrowd_schema_caps: Optional[str]
    is_large: bool
    is_resource: bool
    is_default: bool
    is_broken: bool
    seacrowd_version: str
    source_version: str
    citation: str
    description: str
    homepage: str
    license: str

    _ds_module: datasets.load.DatasetModule = field(repr=False)
    _py_module: ModuleType = field(repr=False)
    _ds_cls: type = field(repr=False)

    def get_load_dataset_kwargs(
        self,
        **extra_load_dataset_kwargs,
    ):
        return {
            "path": self.script,
            "name": self.config.name,
            **extra_load_dataset_kwargs,
        }

    def load_dataset(
        self,
        trust_remote_code=True,
        **extra_load_dataset_kwargs,
    ):
        return datasets.load_dataset(
            path=self.script,
            name=self.config.name,
            trust_remote_code=trust_remote_code,
            **extra_load_dataset_kwargs,
        )

    def get_metadata(self, **extra_load_dataset_kwargs):
        if not self.is_seacrowd_schema:
            raise ValueError("only supported for seacrowd schemas")
        dsd = self.load_dataset(**extra_load_dataset_kwargs)
        split_metas = {}
        for split, ds in dsd.items():
            meta = SCHEMA_TO_METADATA_CLS[self.config.schema].from_dataset(ds)
            split_metas[split] = meta
        return split_metas


def default_is_keeper(metadata: SEACrowdMetadata) -> bool:
    return not metadata.is_large and not metadata.is_resource and metadata.is_seacrowd_schema

class SEACrowdConfigHelper:
    """
    Handles creating and filtering SEACrowdMetadata instances.
    """

    def __init__(
        self,
        helpers: Optional[Iterable[SEACrowdMetadata]] = None,
        keep_broken: bool = False,
        trust_remote_code: bool = True,
    ):

        path_to_here = pathlib.Path(__file__).parent.absolute()
        self.path_to_sea_datasets = (path_to_here / "sea_datasets").resolve()
        self.dataloader_scripts = sorted(
            self.path_to_sea_datasets.glob(os.path.join("*", "*.py"))
        )
        self.dataloader_scripts = [
            el for el in self.dataloader_scripts if el.name != "__init__.py"
        ]

        # if helpers are passed in, just attach and go
        if helpers is not None:
            if keep_broken:
                self._helpers = helpers
            else:
                self._helpers = [helper for helper in helpers if not helper.is_broken]
            return

        # otherwise, create all helpers available in package
        helpers = []
        for dataloader_script in tqdm(self.dataloader_scripts):
            if dataloader_script.stem != dataloader_script.parent.stem:
                continue
            dataset_name = dataloader_script.stem
            py_module = SourceFileLoader(
                dataset_name, dataloader_script.as_posix()
            ).load_module()
            ds_module = datasets.load.dataset_module_factory(
                dataloader_script.as_posix(), trust_remote_code=trust_remote_code,
            )
            ds_cls = datasets.load.import_main_class(ds_module.module_path)

            for config in ds_cls.BUILDER_CONFIGS:

                is_seacrowd_schema = config.schema.startswith("seacrowd")
                if is_seacrowd_schema:
                    seacrowd_schema_caps = '_'.join(config.schema.split("_")[1:]).upper()
                    tasks = SCHEMA_TO_TASKS[seacrowd_schema_caps] & set(
                        py_module._SUPPORTED_TASKS
                    )
                else:
                    tasks = py_module._SUPPORTED_TASKS
                    seacrowd_schema_caps = None

                helpers.append(
                    SEACrowdMetadata(
                        script=dataloader_script.as_posix(),
                        dataset_name=dataset_name,
                        tasks=tasks,
                        languages=py_module._LANGUAGES,
                        config=config,
                        is_local=py_module._LOCAL,
                        is_seacrowd_schema=is_seacrowd_schema,
                        seacrowd_schema_caps=seacrowd_schema_caps,
                        is_large=config.name in _LARGE_CONFIG_NAMES,
                        is_resource=config.name in _RESOURCE_CONFIG_NAMES,
                        is_default=config.name == ds_cls.DEFAULT_CONFIG_NAME,
                        is_broken=config.name in _CURRENTLY_BROKEN_NAMES,
                        seacrowd_version=py_module._SEACROWD_VERSION,
                        source_version=py_module._SOURCE_VERSION,
                        citation=py_module._CITATION,
                        description=py_module._DESCRIPTION,
                        homepage=py_module._HOMEPAGE,
                        license=py_module._LICENSE,
                        _ds_module=ds_module,
                        _py_module=py_module,
                        _ds_cls=ds_cls,
                    )
                )

        if keep_broken:
            self._helpers = helpers
        else:
            self._helpers = [helper for helper in helpers if not helper.is_broken]

    @property
    def available_dataset_names(self) -> List[str]:
        return sorted(list(set([helper.dataset_name for helper in self])))
    
    @property
    def available_config_names(self) -> List[str]:
        return sorted(list(set([helper.config.name for helper in self])))

    def for_dataset(self, dataset_name: str) -> "SEACrowdConfigHelper":
        helpers = [helper for helper in self if helper.dataset_name == dataset_name]
        if len(helpers) == 0:
            raise ValueError(f"no helper with helper.dataset_name = {dataset_name}")
        return SEACrowdConfigHelper(helpers=helpers)

    def for_config_name(self, config_name: str) -> "SEACrowdMetadata":
        helpers = [helper for helper in self if helper.config.name == config_name]
        if len(helpers) == 0:
            raise ValueError(f"no helper with helper.config.name = {config_name}")
        if len(helpers) > 1:
            raise ValueError(
                f"multiple helpers with helper.config.name = {config_name}"
            )
        return helpers[0]

    def default_for_dataset(self, dataset_name: str) -> "SEACrowdMetadata":
        helpers = [
            helper
            for helper in self
            if helper.is_default and helper.dataset_name == dataset_name
        ]
        assert len(helpers) == 1
        return helpers[0]

    def filtered(
        self, is_keeper: Callable[[SEACrowdMetadata], bool]
    ) -> "SEACrowdConfigHelper":
        """Return dataset config helpers that match is_keeper."""
        return SEACrowdConfigHelper(
            helpers=[helper for helper in self if is_keeper(helper)]
        )

    def __repr__(self):
        # return "\n\n".join([helper.__repr__() for helper in self])
        return f"SEACrowd Config Helper. Datasets include: {self.available_dataset_names}."

    def __str__(self):
        return self.__repr__()
        # # return f"SEACrowd Config Helper. Datasets include: {self.available_dataset_names}."
        # return "SEACrowd Config Helper."

    def __iter__(self):
        for helper in self._helpers:
            yield helper

    def __len__(self):
        return len(self._helpers)

    def __getitem__(self, key):
        if isinstance(key, slice):
            start, stop, step = key.indices(len(self))
            return SEACrowdConfigHelper(
                helpers=[self._helpers[ii] for ii in range(start, stop, step)]
            )
        elif isinstance(key, int):
            if key < 0:  # Handle negative indices
                key += len(self)
            if key < 0 or key >= len(self):
                raise IndexError(f"The index ({key}) is out of range.")
            return self._helpers[key]
        else:
            raise TypeError("Invalid argument type.")
    
    def list_datasets(self, with_config=False):
        name_to_schema = {}
        for helper in self:
            if helper.dataset_name not in name_to_schema:
                name_to_schema[helper.dataset_name] = []
            name_to_schema[helper.dataset_name].append(helper.config.name)
        if not with_config:
            return list(name_to_schema.keys())
        else:
            return name_to_schema
    
    def load_dataset(self, dataset_name, schema='seacrowd'):
        try:        
            for helper in sorted(self.filtered(
                    lambda x: (
                        (dataset_name == x.dataset_name) and 
                        (x.is_seacrowd_schema if schema == 'seacrowd' else not x.is_seacrowd_schema)
                    )
                ), key=lambda x: len(x.config.name)):
                return helper.load_dataset()
        except:
            raise ValueError(f"Couldn't find dataset with name=`{dataset_name}` and schema=`{schema}`")

    def load_datasets(self, dataset_names, schema='seacrowd'):
        return {
            helper.config.name: helper.load_dataset()
            for helper in self.filtered(
                lambda x: (
                    (x.dataset_name in dataset_names) and 
                    (x.is_seacrowd_schema if schema == 'seacrowd' else not x.is_seacrowd_schema)
                )
            )
       }
    
    def load_dataset_by_config_name(self, config_name, schema='seacrowd'):
        try:        
            for helper in sorted(self.filtered(
                    lambda x: (
                        (config_name == x.config.name) and 
                        (x.is_seacrowd_schema if schema == 'seacrowd' else not x.is_seacrowd_schema)
                    )
                ), key=lambda x: len(x.config.name)):
                return helper.load_dataset()
        except:
            raise ValueError(f"Couldn't find dataset with config.name=`{config_name}` and schema=`{schema}`")

    def load_datasets_by_config_names(self, config_names, schema='seacrowd'):
        return {
            helper.config.name: helper.load_dataset()
            for helper in self.filtered(
                lambda x: (
                    (x.config.name in config_names) and 
                    (x.is_seacrowd_schema if schema == 'seacrowd' else not x.is_seacrowd_schema)
                )
            )
       }
        
    def list_benchmarks(self):
        return list(BENCHMARK_DICT.keys())

    def load_benchmark(self, benchmark_name):
        return {
            helper.config.name: helper.load_dataset()
            for helper in self.filtered(
                lambda x: (
                    x.config.name in BENCHMARK_DICT[benchmark_name]
                )
            )
        }

# Metadata Helper
@dataclass
class MetaDict:
    data: dict = None
    
class SEACrowdMetadataHelper:
    """
    Handles creating and filtering SEACrowdMetadata instances.
    """

    def __init__(
        self,
        meta_df: Optional[pd.DataFrame] = None,
        keep_broken: bool = False
    ):
        # Load Config Helper
        self._conhelps = SEACrowdConfigHelper()
        
        # if meta_df are passed in, just attach and go
        if meta_df is not None:
            if keep_broken:
                self._meta_df = meta_df
            else:
                self._meta_df = meta_df[~meta_df.is_broken]
            return
        
        # Load Metadata
        self._meta_df = pd.read_csv('https://docs.google.com/spreadsheets/d/17o83IvWxmtGLYridZis0nEprHhsZIMeFtHGtXV35h6M/export?format=csv&gid=879729812', skiprows=1)
        self._meta_df = self._meta_df[self._meta_df['Implemented'] != 0].rename({
            'No.': 'id', 'Name': 'name', 'Subsets': 'subsets', 'Link': 'source_link', 'Description': 'description',
            'HF Link': 'hf_link', 'License': 'license', 'Year': 'year', 'Collection Style': 'collection_style',
            'Language': 'language', 'Dialect': 'dialect', 'Domain': 'domain', 'Form': 'modality', 'Tasks': 'tasks',
            'Volume': 'volume', 'Unit': 'unit', 'Ethical Risks': 'ethical_risk', 'Provider': 'provider',
            'Paper Title': 'paper_title', 'Paper Link': 'paper_link', 'Access': 'access', 'Derived From': 'derived_from', 
            'Test Split': 'is_splitted', 'Notes': 'notes', 'Dataloader': 'dataloader', 'Implemented': 'implemented'
        }, axis=1)
        self._meta_df['is_splitted'] = self._meta_df['is_splitted'].apply(lambda x: True if x =='Yes' else False)

        # Merge Metadata with Config
        name_to_meta_map = {}
        for cfg_meta in self._conhelps:
            # Assign metadata to meta dataframe
            self._meta_df.loc[self._meta_df.dataloader == cfg_meta.dataset_name, [
                'is_large', 'is_resource', 'is_default', 'is_broken',
                'is_local', 'citation', 'license', 'homepage', 'tasks'
            ]] = [
                cfg_meta.is_large, cfg_meta.is_resource, cfg_meta.is_default, cfg_meta.is_broken, 
                cfg_meta.is_local, cfg_meta.citation, cfg_meta.license, cfg_meta.homepage, '|'.join([task.value for task in cfg_meta.tasks])
            ]

            if cfg_meta.dataset_name not in name_to_meta_map:
                name_to_meta_map[cfg_meta.dataset_name] = {}
            if cfg_meta.config.schema not in name_to_meta_map[cfg_meta.dataset_name]:
                name_to_meta_map[cfg_meta.dataset_name][cfg_meta.config.schema] = []
            name_to_meta_map[cfg_meta.dataset_name][cfg_meta.config.schema].append(cfg_meta)
        
        self._meta_df = self._meta_df.fillna(False)
        for dset_name in name_to_meta_map.keys():
            self._meta_df.loc[self._meta_df.dataloader == dset_name, 'metadata'] = MetaDict(data=name_to_meta_map[dset_name])

        if not keep_broken:
            self._meta_df = self._meta_df[~self._meta_df.is_broken]
    
    def filtered(
        self, is_keeper: Callable[[], bool]
    ) -> "SEACrowdConfigHelper":
        """Return dataset config helpers that match is_keeper."""
        meta_df = self._meta_df[self._meta_df.apply(is_keeper, axis=1, reduce=True)]
        return SEACrowdMetadataHelper(meta_df=meta_df)

    def filter_and_load(
        self, is_keeper: Callable[[], bool]
    ) -> "Dict<str, Dataset>":
        """Return dataset that match is_keeper."""
        filtered_helper = self.filtered(is_keeper)
        for metas in filtered_helper._meta_df.metadata:
            if schema in metas.data:
                for meta in metas.data[schema]:
                    if len(meta.languages) > 1:
                        if lang in meta.config.name:
                            datasets[meta.config.name] = meta.load_dataset()
                    else:
                        datasets[meta.config.name] = meta.load_dataset()
                
    @property
    def available_dataset_names(self) -> List[str]:
        return sorted(self._meta_df.name)

    def __repr__(self):
        return self._meta_df.to_string()

    def __str__(self):
        # return self.__repr__()
        return "test"

    def __iter__(self):
        for row in self._meta_df.iterrows():
            yield row

    def __len__(self):
        return len(self._meta_df)
    
###
# SEACrowd Interface
###

def list_datasets(with_config=False):
    conhelps = SEACrowdConfigHelper()
    return conhelps.list_datasets(with_config=with_config)

def load_dataset(dataset_name, schema='seacrowd'):
    conhelps = SEACrowdConfigHelper()
    return conhelps.load_dataset(dataset_name=dataset_name, schema=schema)

def load_datasets(dataset_names, schema='seacrowd'):
    conhelps = SEACrowdConfigHelper()
    return conhelps.load_datasets(dataset_names=dataset_names, schema=schema)

def list_benchmarks():
    conhelps = SEACrowdConfigHelper()
    return conhelps.list_benchmarks()

def load_benchmark(benchmark_name):
    conhelps = SEACrowdConfigHelper()
    return conhelps.load_benchmark(benchmark_name=benchmark_name)

if __name__ == "__main__":
    print(f'LIST DATASETS')
    dset_names = list_datasets()
    print(dset_names[:10])
    print()

    print(f'LOAD DATASET `{dset_names[1]}`')
    dset = load_dataset(dset_names[1])
    print(dset)
    print()
    
    print(f'LOAD DATASETS [{dset_names[1:4]}]')
    dsets = load_datasets(dset_names[1:4])
    print(dsets)
    print()
    
    print(f'LIST BENCHMARKS')
    benchmark_names = list_benchmarks()
    print(benchmark_names[:3])
    print()

    print(f'LOAD BENCHMARK `{benchmark_names[0]}`')
    benchmark_dsets = load_benchmark(benchmark_names[0])
    print(benchmark_dsets)
    print()


import json
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@article{WILLIAM2020106231,
title = "CLICK-ID: A novel dataset for Indonesian clickbait headlines",
journal = "Data in Brief",
volume = "32",
pages = "106231",
year = "2020",
issn = "2352-3409",
doi = "https://doi.org/10.1016/j.dib.2020.106231",
url = "http://www.sciencedirect.com/science/article/pii/S2352340920311252",
author = "Andika William and Yunita Sari",
keywords = "Indonesian, Natural Language Processing, News articles, Clickbait, Text-classification",
abstract = "News analysis is a popular task in Natural Language Processing (NLP). In particular, the problem of clickbait in news analysis has gained attention in recent years [1, 2]. However, the majority of the tasks has been focused on English news, in which there is already a rich representative resource. For other languages, such as Indonesian, there is still a lack of resource for clickbait tasks. Therefore, we introduce the CLICK-ID dataset of Indonesian news headlines extracted from 12 Indonesian online news publishers. It is comprised of 15,000 annotated headlines with clickbait and non-clickbait labels. Using the CLICK-ID dataset, we then developed an Indonesian clickbait classification model achieving favourable performance. We believe that this corpus will be useful for replicable experiments in clickbait detection or other experiments in NLP areas."
}
"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "id_clickbait"
_DESCRIPTION = """\
The CLICK-ID dataset is a collection of Indonesian news headlines that was collected from 12 local online news
publishers; detikNews, Fimela, Kapanlagi, Kompas, Liputan6, Okezone, Posmetro-Medan, Republika, Sindonews, Tempo,
Tribunnews, and Wowkeren. This dataset is comprised of mainly two parts; (i) 46,119 raw article data, and (ii)
15,000 clickbait annotated sample headlines. Annotation was conducted with 3 annotator examining each headline.
Judgment were based only on the headline. The majority then is considered as the ground truth. In the annotated
sample, our annotation shows 6,290 clickbait and 8,710 non-clickbait.
"""
_HOMEPAGE = "https://www.sciencedirect.com/science/article/pii/S2352340920311252#!"
_LICENSE = "Creative Commons Attribution 4.0 International"
_URLS = {
    _DATASETNAME: "https://prod-dcd-datasets-cache-zipfiles.s3.eu-west-1.amazonaws.com/k42j7x2kpn-1.zip",
}
_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IdClickbait(datasets.GeneratorBasedBuilder):
    """The CLICK-ID dataset is a collection of Indonesian news headlines that was collected from 12 local online news, annotated with a label whether each is a clickbait"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="id_clickbait_source",
            version=SOURCE_VERSION,
            description="CLICK-ID source schema",
            schema="source",
            subset_id="id_clickbait",
        ),
        SEACrowdConfig(
            name="id_clickbait_seacrowd_text",
            version=SEACROWD_VERSION,
            description="CLICK-ID Nusantara schema",
            schema="seacrowd_text",
            subset_id="id_clickbait",
        ),
    ]

    DEFAULT_CONFIG_NAME = "id_clickbait_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({"title": datasets.Value("string"), "label": datasets.Value("string"), "label_score": datasets.Value("int8")})
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(["non-clickbait", "clickbait"])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        # Dataset does not have predetermined split, putting all as TRAIN
        urls = _URLS[_DATASETNAME]
        base_dir = Path(dl_manager.download_and_extract(urls)) / "annotated" / "combined" / "json"
        data_files = {"train": base_dir / "main.json"}

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_files["train"],
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        # Dataset does not have row id, using python enumeration.
        data = json.load(open(filepath, "r"))

        if self.config.schema == "source":
            for row_index, row in enumerate(data):
                ex = {
                    "title": row["title"],
                    "label": row["label"],
                    "label_score": row["label_score"],
                }
                yield row_index, ex

        elif self.config.schema == "seacrowd_text":
            for row_index, row in enumerate(data):
                ex = {
                    "id": str(row_index),
                    "text": row["title"],
                    "label": row["label"],
                }
                yield row_index, ex



import os
from pathlib import Path
from typing import List

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@article{FrogStorytelling,
  author="Moeljadi, David",
  title="Usage of Indonesian Possessive Verbal Predicates : A Statistical Analysis Based on Storytelling Survey",
  journal="Tokyo University Linguistic Papers",
  ISSN="1345-8663",
  publisher="東京大学大学院人文社会系研究科・文学部言語学研究室",
  year="2014",
  month="sep",
  volume="35",
  number="",
  pages="155-176",
  URL="https://ci.nii.ac.jp/naid/120005525793/en/",
  DOI="info:doi/10.15083/00027472",
}
"""
_DATASETNAME = "id_frog_story"
_DESCRIPTION = """\
Indonesian Frog Storytelling Corpus
Indonesian written and spoken corpus, based on the twenty-eight pictures. (http://compling.hss.ntu.edu.sg/who/david/corpus/pictures.pdf)
"""
_HOMEPAGE = "https://github.com/matbahasa/corpus-frog-storytelling"
_LANGUAGES = ["ind"]
_LICENSE = "Creative Commons Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)"
_LOCAL = False
_URLS = {
    _DATASETNAME: "https://github.com/matbahasa/corpus-frog-storytelling/archive/refs/heads/master.zip",
}
_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IdFrogStory(datasets.GeneratorBasedBuilder):
    """IdFrogStory contains 13 spoken datasets and 11 written datasets"""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="id_frog_story_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="IdFrogStory source schema",
            schema="source",
            subset_id="id_frog_story",
        ),
        SEACrowdConfig(
            name="id_frog_story_seacrowd_ssp",
            version=datasets.Version(_SEACROWD_VERSION),
            description="IdFrogStory Nusantara schema",
            schema="seacrowd_ssp",
            subset_id="id_frog_story",
        ),
    ]

    DEFAULT_CONFIG_NAME = "id_frog_story_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_ssp":
            features = schemas.self_supervised_pretraining.features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        base_path = Path(dl_manager.download_and_extract(urls)) / "corpus-frog-storytelling-master" / "data"
        spoken_path = base_path / "spoken"
        written_path = base_path / "written"

        data = []
        for spoken_file_name in sorted(os.listdir(spoken_path)):
            spoken_file_path = spoken_path / spoken_file_name
            if os.path.isfile(spoken_file_path):
                with open(spoken_file_path, "r") as fspoken:
                    data.extend(fspoken.read().strip("\n").split("\n\n"))

        for written_file_name in sorted(os.listdir(written_path)):
            written_file_path = written_path / written_file_name
            if os.path.isfile(written_file_path):
                with open(written_file_path, "r") as fwritten:
                    data.extend(fwritten.read().strip("\n").split("\n\n"))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "data": data,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, data: List, split: str):
        if self.config.schema == "source":
            for index, row in enumerate(data):
                ex = {
                    "id": index,
                    "text": row
                }
                yield index, ex
        elif self.config.schema == "seacrowd_ssp":
            for index, row in enumerate(data):
                ex = {
                    "id": index,
                    "text": row
                }
                yield index, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{putri2021abusive,
  title={Abusive language and hate speech detection for Javanese and Sundanese languages in tweets: Dataset and preliminary study},
  author={Putri, Shofianina Dwi Ananda and Ibrohim, Muhammad Okky and Budi, Indra},
  booktitle={2021 11th International Workshop on Computer Science and Engineering, WCSE 2021},
  pages={461--465},
  year={2021},
  organization={International Workshop on Computer Science and Engineering (WCSE)},
  abstract={Indonesia’s demography as an archipelago with lots of tribes and local languages added variances in their communication style. Every region in Indonesia has its own distinct culture, accents, and languages. The demographical condition can influence the characteristic of the language used in social media, such as Twitter. It can be found that Indonesian uses their own local language for communicating and expressing their mind in tweets. Nowadays, research about identifying hate speech and abusive language has become an attractive and developing topic. Moreover, the research related to Indonesian local languages still rarely encountered. This paper analyzes the use of machine learning approaches such as Naïve Bayes (NB), Support Vector Machine (SVM), and Random Forest Decision Tree (RFDT) in detecting hate speech and abusive language in Sundanese and Javanese as Indonesian local languages. The classifiers were used with the several term weightings features, such as word n-grams and char n-grams. The experiments are evaluated using the F-measure. It achieves over 60 % for both local languages.}
}
"""
_DATASETNAME = "local_id_abusive"

_DESCRIPTION = """\
This dataset is for abusive and hate speech detection, using Twitter text containing Javanese and Sundanese words.

(from the publication source)
The Indonesian local language dataset collection was conducted using Twitter search API to collect the tweets and then
implemented using Tweepy Library. The tweets were collected using queries from the list of abusive words in Indonesian
tweets. The abusive words were translated into local Indonesian languages, which are Javanese and Sundanese. The
translated words are then used as queries to collect tweets containing Indonesian and local languages. The translation
process involved native speakers for each local language. The crawling process has collected a total of more than 5000
tweets. Then, the crawled data were filtered to get tweets that contain local’s vocabulary and/or sentences in Javanese
and Sundanese. Next, after the filtering process, the data will be labeled whether the tweets are labeled as hate speech
and abusive language or not.
"""

_HOMEPAGE = "https://github.com/Shofianina/local-indonesian-abusive-hate-speech-dataset"

_LICENSE = "Unknown"

_LANGUAGES = ["jav", "sun"]
_LOCAL = False

_URLS = {
    _DATASETNAME: {
        "jav": "https://raw.githubusercontent.com/Shofianina/local-indonesian-abusive-hate-speech-dataset/main/Javanese.csv",
        "sun": "https://raw.githubusercontent.com/Shofianina/local-indonesian-abusive-hate-speech-dataset/main/Sundanese.csv",
    }
}

_SUPPORTED_TASKS = [Tasks.ASPECT_BASED_SENTIMENT_ANALYSIS]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class LocalIDAbusive(datasets.GeneratorBasedBuilder):
    """Local ID Abusive is a dataset for abusive and hate speech detection, using Twitter text containing Javanese and
    Sundanese words."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="local_id_abusive_jav_source",
            version=SOURCE_VERSION,
            description="local_id_abusive source schema Javanese",
            schema="source",
            subset_id="local_id_abusive_jav",
        ),
        SEACrowdConfig(
            name="local_id_abusive_sun_source",
            version=SOURCE_VERSION,
            description="local_id_abusive source schema Sundanese",
            schema="source",
            subset_id="local_id_abusive_sun",
        ),
        SEACrowdConfig(
            name="local_id_abusive_jav_seacrowd_text_multi",
            version=SEACROWD_VERSION,
            description="local_id_abusive Nusantara schema Javanese",
            schema="seacrowd_text_multi",
            subset_id="local_id_abusive_jav",
        ),
        SEACrowdConfig(
            name="local_id_abusive_sun_seacrowd_text_multi",
            version=SEACROWD_VERSION,
            description="local_id_abusive Nusantara schema Sundanese",
            schema="seacrowd_text_multi",
            subset_id="local_id_abusive_sun",
        ),
    ]

    DEFAULT_CONFIG_NAME = "local_id_abusive_jav_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "isi_tweet": datasets.Value("string"),
                    "uk": datasets.Value("bool"),
                    "hs": datasets.Value("bool"),
                }
            )
        elif self.config.schema == "seacrowd_text_multi":
            features = schemas.text_multi_features([0, 1])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        lang = self.config.name.split("_")[3]
        urls = _URLS[_DATASETNAME][lang]
        data_dir = dl_manager.download_and_extract(urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # Whatever you put in gen_kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": data_dir,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        df = pd.read_csv(filepath, sep=",", encoding="ISO-8859-1").reset_index()
        for i, row in enumerate(df.itertuples()):
            if self.config.schema == "source":
                example = {"isi_tweet": row.isi_tweet, "uk": row.uk, "hs": row.hs}
                yield i, example
            elif self.config.schema == "seacrowd_text_multi":
                example = {
                    "id": str(i),
                    "text": row.isi_tweet,
                    "labels": [row.uk, row.hs],
                }
                yield i, example



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import (DEFAULT_SEACROWD_VIEW_NAME,
                                       DEFAULT_SOURCE_VIEW_NAME, Tasks)

_LOCAL = False

_DATASETNAME = "nusax_senti"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind", "ace", "ban", "bjn", "bbc", "bug", "jav", "mad", "min", "nij", "sun", "eng"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_CITATION = """\
@misc{winata2022nusax,
      title={NusaX: Multilingual Parallel Sentiment Dataset for 10 Indonesian Local Languages},
      author={Winata, Genta Indra and Aji, Alham Fikri and Cahyawijaya,
      Samuel and Mahendra, Rahmad and Koto, Fajri and Romadhony,
      Ade and Kurniawan, Kemal and Moeljadi, David and Prasojo,
      Radityo Eko and Fung, Pascale and Baldwin, Timothy and Lau,
      Jey Han and Sennrich, Rico and Ruder, Sebastian},
      year={2022},
      eprint={2205.15960},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""

_DESCRIPTION = """\
NusaX is a high-quality multilingual parallel corpus that covers 12 languages, Indonesian, English, and 10 Indonesian local languages, namely Acehnese, Balinese, Banjarese, Buginese, Madurese, Minangkabau, Javanese, Ngaju, Sundanese, and Toba Batak.

NusaX-Senti is a 3-labels (positive, neutral, negative) sentiment analysis dataset for 10 Indonesian local languages + Indonesian and English.
"""

_HOMEPAGE = "https://github.com/IndoNLP/nusax/tree/main/datasets/sentiment"

_LICENSE = "Creative Commons Attribution Share-Alike 4.0 International"

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_URLS = {
    "train": "https://raw.githubusercontent.com/IndoNLP/nusax/main/datasets/sentiment/{lang}/train.csv",
    "validation": "https://raw.githubusercontent.com/IndoNLP/nusax/main/datasets/sentiment/{lang}/valid.csv",
    "test": "https://raw.githubusercontent.com/IndoNLP/nusax/main/datasets/sentiment/{lang}/test.csv",
}


def seacrowd_config_constructor(lang, schema, version):
    """Construct SEACrowdConfig with nusax_senti_{lang}_{schema} as the name format"""
    if schema != "source" and schema != "seacrowd_text":
        raise ValueError(f"Invalid schema: {schema}")

    if lang == "":
        return SEACrowdConfig(
            name="nusax_senti_{schema}".format(schema=schema),
            version=datasets.Version(version),
            description="nusax_senti with {schema} schema for all 12 languages".format(schema=schema),
            schema=schema,
            subset_id="nusax_senti",
        )
    else:
        return SEACrowdConfig(
            name="nusax_senti_{lang}_{schema}".format(lang=lang, schema=schema),
            version=datasets.Version(version),
            description="nusax_senti with {schema} schema for {lang} language".format(lang=lang, schema=schema),
            schema=schema,
            subset_id="nusax_senti",
        )


LANGUAGES_MAP = {
    "ace": "acehnese",
    "ban": "balinese",
    "bjn": "banjarese",
    "bug": "buginese",
    "eng": "english",
    "ind": "indonesian",
    "jav": "javanese",
    "mad": "madurese",
    "min": "minangkabau",
    "nij": "ngaju",
    "sun": "sundanese",
    "bbc": "toba_batak",
}


class NusaXSenti(datasets.GeneratorBasedBuilder):
    """NusaX-Senti is a 3-labels (positive, neutral, negative) sentiment analysis dataset for 10 Indonesian local languages + Indonesian and English."""

    BUILDER_CONFIGS = (
        [seacrowd_config_constructor(lang, "source", _SOURCE_VERSION) for lang in LANGUAGES_MAP]
        + [seacrowd_config_constructor(lang, "seacrowd_text", _SEACROWD_VERSION) for lang in LANGUAGES_MAP]
        + [seacrowd_config_constructor("", "source", _SOURCE_VERSION), seacrowd_config_constructor("", "seacrowd_text", _SEACROWD_VERSION)]
    )

    DEFAULT_CONFIG_NAME = "nusax_senti_ind_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "label": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(["negative", "neutral", "positive"])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        if self.config.name == "nusax_senti_source" or self.config.name == "nusax_senti_seacrowd_text":
            # Load all 12 languages
            train_csv_path = dl_manager.download_and_extract([_URLS["train"].format(lang=LANGUAGES_MAP[lang]) for lang in LANGUAGES_MAP])
            validation_csv_path = dl_manager.download_and_extract([_URLS["validation"].format(lang=LANGUAGES_MAP[lang]) for lang in LANGUAGES_MAP])
            test_csv_path = dl_manager.download_and_extract([_URLS["test"].format(lang=LANGUAGES_MAP[lang]) for lang in LANGUAGES_MAP])
        else:
            lang = self.config.name[12:15]
            train_csv_path = Path(dl_manager.download_and_extract(_URLS["train"].format(lang=LANGUAGES_MAP[lang])))
            validation_csv_path = Path(dl_manager.download_and_extract(_URLS["validation"].format(lang=LANGUAGES_MAP[lang])))
            test_csv_path = Path(dl_manager.download_and_extract(_URLS["test"].format(lang=LANGUAGES_MAP[lang])))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": train_csv_path},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": validation_csv_path},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": test_csv_path},
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        if self.config.schema != "source" and self.config.schema != "seacrowd_text":
            raise ValueError(f"Invalid config: {self.config.name}")

        if self.config.name == "nusax_senti_source" or self.config.name == "nusax_senti_seacrowd_text":
            ldf = []
            for fp in filepath:
                ldf.append(pd.read_csv(fp))
            df = pd.concat(ldf, axis=0, ignore_index=True).reset_index()
            # Have to use index instead of id to avoid duplicated key
            df = df.drop(columns=["id"]).rename(columns={"index": "id"})
        else:
            df = pd.read_csv(filepath).reset_index()

        for row in df.itertuples():
            ex = {"id": str(row.id), "text": row.text, "label": row.label}
            yield row.id, ex



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import TASK_TO_SCHEMA, Licenses, Tasks

_CITATION = """\
@misc{nomoto2018melayustandardlisan,
	author = {Hiroki Nomoto},
	title = {Korpus Variasi Bahasa Melayu: Standard Lisan},
	year = {2018},
	url = {https://github.com/matbahasa/Melayu_Standard_Lisan}
}
"""

_DATASETNAME = "melayu_standard_lisan"


_DESCRIPTION = """\
Korpus Variasi Bahasa Melayu: Standard Lisan is a language corpus sourced from monologues of various melayu folklores.
"""


_HOMEPAGE = "https://github.com/matbahasa/Melayu_Standard_Lisan"


_LANGUAGES = ["zlm"]

_LICENSE = Licenses.CC_BY_4_0.value

_LOCAL = False

_URLS = {
    "kl201701": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201701.txt",
    "kl201702": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201702.txt",
    "kl201703": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201703.txt",
    "kl201704": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201704.txt",
    "kl201705": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201705.txt",
    "kl201706": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201706.txt",
    "kl201707": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201707.txt",
    "kl201708": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201708.txt",
    "kl201709": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201709.txt",
    "kl201710": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201710.txt",
    "kl201711": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201711.txt",
    "kl201712": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201712.txt",
    "kl201713": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201713.txt",
    "kl201714": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201714.txt",
    "kl201715": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201715.txt",
    "kl201716": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201716.txt",
    "kl201717": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201717.txt",
    "kl201718": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201718.txt",
    "kl201719": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201719.txt",
    "kl201720": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201720.txt",
    "kl201721": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201721.txt",
    "kl201722": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201722.txt",
    "kl201723": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201723.txt",
    "kl201724": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201724.txt",
    "kl201725": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201725.txt",
    "kl201726": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201726.txt",
    "kl201727": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201727.txt",
    "kl201728": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201728.txt",
    "kl201729": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201729.txt",
    "kl201730": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201730.txt",
    "kl201731": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201731.txt",
    "kl201732": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201732.txt",
    "kl201733": "https://raw.githubusercontent.com/matbahasa/Melayu_Standard_Lisan/master/KL201733.txt",
}

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class MelayuStandardLisan(datasets.GeneratorBasedBuilder):
    """Korpus Variasi Bahasa Melayu:
    Standard Lisan is a language corpus sourced from monologues of various melayu folklores."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = TASK_TO_SCHEMA[_SUPPORTED_TASKS[0]].lower()

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.self_supervised_pretraining.features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        urls = [_URLS[key] for key in _URLS.keys()]
        data_path = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_path[0], "split": "train", "other_path": data_path[1:]},
            )
        ]

    def _generate_examples(self, filepath: Path, split: str, other_path: List) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        filepaths = [filepath] + other_path
        data = []
        for filepath in filepaths:
            with open(filepath, "r") as f:
                data.append(" ".join([line.rstrip() for line in f.readlines()]))

        for id, text in enumerate(data):
            yield id, {"id": id, "text": text}



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import SCHEMA_TO_FEATURES, TASK_TO_SCHEMA, Tasks

# TODO: Add BibTeX citation
_CITATION = r"""\
@article{oco2015witchebelles,
  author    = {Oco, Nathaniel and Fajutagana, Raymart and Lim, Christine Mae and Mi{\~n}on, Judi Diane and Morano, Julie-Ann and Tinoco, Ryan Christian},
  title     = {Witchebelles Anata Magcharot kay Mudra na Nagsusuba si Akech: Developing a Rule-based Unidirectional Beki Lingo to Filipino Translator},
  journal   = {Journal of Sciences, Technology and Arts Research},
  volume    = {1},
  number    = {1},
  year      = {2015}
}
"""

_LOCAL = False
_LANGUAGES = ["fil"]
_DATASETNAME = "filipino_gay_lang"
_DESCRIPTION = """\
The dataset contains 4000+ Filipino tweets in gay language or lingo also called swardspeak in slang terminology.
The tweet dataset was collected from February 2013 to November 2014 using the following commonly used gay words as filters: jinet ("hot"), ditey ("here"), imbyerna ("annoying"), etc.
The original paper makes use of the corpus to develop a gay language translator to understand the meaning of phrases using gay words in Filipino.
"""

_HOMEPAGE = "https://github.com/imperialite/Philippine-Languages-Online-Corpora/tree/master/Tweets/Gay%20language"
_LICENSE = "CC-BY-SA 4.0"
_URLS = {
    "gl_01": "https://github.com/imperialite/Philippine-Languages-Online-Corpora/raw/master/Tweets/Gay%20language/gl%20-%2001.xlsx",
    "gl_02": "https://github.com/imperialite/Philippine-Languages-Online-Corpora/raw/master/Tweets/Gay%20language/gl%20-%2002.xlsx",
    "gl_03": "https://github.com/imperialite/Philippine-Languages-Online-Corpora/raw/master/Tweets/Gay%20language/gl%20-%2003.xlsx",
}

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class FilipinoGayLangDataset(datasets.GeneratorBasedBuilder):
    """This dataset contains 4000+ Filipino tweets in gay lingo/Beki/Swardspeak."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = TASK_TO_SCHEMA[_SUPPORTED_TASKS[0]].lower()

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({"index": datasets.Value("string"), "text": datasets.Value("string")})
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = SCHEMA_TO_FEATURES[self.SEACROWD_SCHEMA_NAME.upper()]

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        data_files = {
            "gl_01": Path(dl_manager.download(_URLS["gl_01"])),
            "gl_02": Path(dl_manager.download(_URLS["gl_02"])),
            "gl_03": Path(dl_manager.download(_URLS["gl_03"])),
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": [data_files["gl_01"], data_files["gl_02"], data_files["gl_03"]], "split": "train"},
            )
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        df = pd.concat((pd.read_excel(file) for file in filepath), ignore_index=True).reset_index()

        if self.config.schema == "source":
            for row in df.itertuples():
                ex = {"index": str(row.index), "text": row.message}
                yield row.index, ex
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            for row in df.itertuples():
                ex = {"id": str(row.index), "text": row.message}
                yield row.index, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



import os
from pathlib import Path
from typing import Dict, List, Tuple
from seacrowd.utils.constants import Tasks
from seacrowd.utils import schemas

import datasets
import json
import xml.etree.ElementTree as ET

from seacrowd.utils.configs import SEACrowdConfig

_CITATION = """\
@article{nuranti2022predicting,
  title={Predicting the Category and the Length of Punishment in Indonesian Courts Based on Previous Court Decision Documents},
  author={Nuranti, Eka Qadri and Yulianti, Evi and Husin, Husna Sarirah},
  journal={Computers},
  volume={11},
  number={6},
  pages={88},
  year={2022},
  publisher={Multidisciplinary Digital Publishing Institute}
}
"""

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "indo_law"

_DESCRIPTION = """\
This study presents predictions of first-level judicial decisions by utilizing a collection of Indonesian court decision documents. 
We propose using multi-level learning, namely, CNN+attention, using decision document sections as features to predict the category and the length of punishment in Indonesian courts. 
Our results demonstrate that the decision document sections that strongly affected the accuracy of the prediction model were prosecution history, facts, legal facts, and legal considerations.
"""

_HOMEPAGE = ""

_LICENSE = "Unknown"

_URLS = {
    _DATASETNAME: "https://github.com/ir-nlp-csui/indo-law/zipball/master",
}

_SUPPORTED_TASKS = [Tasks.LEGAL_CLASSIFICATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

class IndoLaw(datasets.GeneratorBasedBuilder):

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    _LABELS = ["pidana-khusus", "pidana-umum"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="indo_law_source",
            version=SOURCE_VERSION,
            description="Indo-Law source schema",
            schema="source",
            subset_id="indo_law",
        ),
        SEACrowdConfig(
            name="indo_law_seacrowd_text",
            version=SEACROWD_VERSION,
            description="Indo-Law Nusantara schema",
            schema="seacrowd_text",
            subset_id="indo_law",
        ),
    ]

    DEFAULT_CONFIG_NAME = "indo_law_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "klasifikasi": datasets.Value("string"),
                    "sub_klasifikasi": datasets.Value("string"),
                    "paragraphs": datasets.Sequence({
                        "tag": datasets.Value("string"),
                        "value": datasets.Value("string"),
                    }),
                }
            )

        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(self._LABELS)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        data_dir = os.path.join(data_dir, "ir-nlp-csui-indo-law-6734033", "dataset")

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        files = os.listdir(filepath)

        results = []

        for file in files:
            data = self._parse_file(os.path.join(filepath, file))
            results.append(data)

        if self.config.schema == "source":
            key = 0
            for result in results:
                example = {
                    "id": result["id"],
                    "klasifikasi": result["klasifikasi"],
                    "sub_klasifikasi": result["klasifikasi"],
                    "paragraphs": [],
                }
                for tag in result["paragraphs"]:
                    example["paragraphs"].append({
                        "tag": tag,
                        "value": result["paragraphs"][tag]
                    })
                yield key, example
                key+=1

        elif self.config.schema == "seacrowd_text":
            key = 0
            for result in results:
                example = {
                    "id": result["id"],
                    "text": json.dumps(result["paragraphs"]),
                    "label": result["klasifikasi"],
                }
                yield key, example
                key+=1

    def _parse_file(self, file_path):
        root = ET.parse(file_path).getroot()

        data = {
            "id": root.attrib["id"],
            "klasifikasi": root.attrib["klasifikasi"],
            "sub_klasifikasi": root.attrib["sub_klasifikasi"],
            "paragraphs": {}
        }

        for child in root:
            data["paragraphs"].update({
                child.tag: child.text
            })

        return data
                




# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{sakti-tts-cocosda-2008,
    title = "Development of HMM-based Indonesian Speech Synthesis",
    author = "Sakti, Sakriani and Maia, Ranniery and Sakai, Shinsuke and Nakamura, Satoshi",
    booktitle = "Proc. Oriental COCOSDA",
    year = "2008",
    pages = "215--220"
    address = "Kyoto, Japan"
}

@inproceedings{sakti-tts-malindo-2010,
    title = "Quality and Intelligibility Assessment of Indonesian HMM-Based Speech Synthesis System",
    author = "Sakti, Sakriani and Sakai, Shinsuke and Isotani, Ryosuke and Kawai, Hisashi and Nakamura, Satoshi",
    booktitle = "Proc. MALINDO",
    year = "2010",
    pages = "51--57"
    address = "Jakarta, Indonesia"
}

@article{sakti-s2st-csl-2013,
    title = "{A-STAR}: Toward Tranlating Asian Spoken Languages",
    author = "Sakti, Sakriani and Paul, Michael and Finch, Andrew and Sakai, Shinsuke and Thang, Tat Vu, and Kimura, Noriyuki 
    and Hori, Chiori and Sumita, Eiichiro and Nakamura, Satoshi and Park, Jun and Wutiwiwatchai, Chai and Xu, Bo and Riza, Hammam 
    and Arora, Karunesh and Luong, Chi Mai and Li, Haizhou",
    journal = "Special issue on Speech-to-Speech Translation, Computer Speech and Language Journal",
    volume = "27",
    number ="2",
    pages = "509--527",
    year = "2013",
    publisher = "Elsevier"
}
"""

_DATASETNAME = "INDspeech_NEWS_TTS"
_LANGUAGES = ["ind"]

_DESCRIPTION = """\
INDspeech_NEWS_TTS is a speech dataset for developing an Indonesian text-to-speech synthesis system. The data was developed by Advanced Telecommunication Research Institute International (ATR) Japan under the the Asian speech translation advanced research (A-STAR) project [Sakti et al., 2013].
"""
_HOMEPAGE = "https://github.com/s-sakti/data_indsp_news_tts"

_LICENSE = "CC-BY-NC-SA 4.0"

_TRAIN_TASKS = {"120": "Orig_trainset_120min.lst", "60": "Orig_trainset_60min.lst", "30": "Orig_trainset_30min.lst", "12": "Orig_trainset_12min.lst", "ZR": "ZRChallenge_trainset.lst"}

_TEST_TASKS = {
    "MOS": "Orig_testset_MOS.lst",
    # "SUS": "Orig_testset_SUS.lst",
    "ZR": "ZRChallenge_testset.lst",
}

_URLS = {"lst_": "https://github.com/s-sakti/data_indsp_news_tts/raw/main/lst/", "speech_": "https://github.com/s-sakti/data_indsp_news_tts/raw/main/speech/", "text_": "https://github.com/s-sakti/data_indsp_news_tts/raw/main/text/orig_transcript"}

_SUPPORTED_TASKS = [Tasks.TEXT_TO_SPEECH]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"
_LOCAL = False


def seacrowd_config_constructor(schema, version, train_task, test_task):

    if schema != "source" and schema != "seacrowd_sptext":
        raise ValueError(f"Invalid schema: {schema}")

    return SEACrowdConfig(
        name="indspeech_news_tts_{tr_task}_{ts_task}_{schema}".format(schema=schema, tr_task=train_task, ts_task=test_task),
        version=datasets.Version(version),
        description="indspeech_news_tts {schema} schema for {tr_task} train and {ts_task} test task".format(schema=schema, tr_task=train_task, ts_task=test_task),
        schema=schema,
        subset_id="indspeech_news_tts_{tr_task}_{ts_task}".format(tr_task=train_task, ts_task=test_task),
    )


class INDspeechNEWSTTS(datasets.GeneratorBasedBuilder):
    """
    Tasks:
    Original = Train [120, 60, 30, 12], Test [MOS, SUS]
    ZR = Train, Test
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = (
        [seacrowd_config_constructor("source", _SOURCE_VERSION, train, test) for train in ["12", "30", "60", "120"] for test in ["MOS"]]
        + [seacrowd_config_constructor("seacrowd_sptext", _SEACROWD_VERSION, train, test) for train in ["12", "30", "60", "120"] for test in ["MOS"]]
        + [seacrowd_config_constructor("source", _SOURCE_VERSION, "ZR", "ZR")]
        + [seacrowd_config_constructor("seacrowd_sptext", _SEACROWD_VERSION, "ZR", "ZR")]
    )

    DEFAULT_CONFIG_NAME = "indspeech_news_tts_120_MOS_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "speaker_id": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "text": datasets.Value("string"),
                    "gender": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        tr_task = self.config.name.split("_")[3]  # [12,30,60,120,ZR]
        ts_task = self.config.name.split("_")[4]  # [MOS, ZR]

        lst_train_dir = Path(dl_manager.download_and_extract(_URLS["lst_"] + _TRAIN_TASKS[tr_task]))
        lst_test_dir = Path(dl_manager.download_and_extract(_URLS["lst_"] + _TEST_TASKS[ts_task]))
        txt_dir = Path(dl_manager.download_and_extract(_URLS["text_"]))
        speech_dir = {"SPK00_" + str(spk).zfill(2) + "00": Path(dl_manager.download_and_extract(_URLS["speech_"] + "SPK00_" + str(spk).zfill(2) + "00.zip") + "/SPK00_" + str(spk).zfill(2) + "00") for spk in range(0, 21)}
        # print(os.listdir(speech_dir["SPK00_1500"]))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # Whatever you put in gen_kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": {"samples": lst_train_dir, "text": txt_dir, "speech": speech_dir},
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": {"samples": lst_test_dir, "text": txt_dir, "speech": speech_dir},
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        samples = open(filepath["samples"], "r").read().splitlines()

        transcripts = {}
        with open(filepath["text"]) as file:
            for line in file:
                key, text = line.replace("\n", "").split("\t")
                transcripts[key] = text

        for key, id in enumerate(samples):
            spk_id, gender, speech_id = id.split("_")
            spk_group = speech_id[:2]

            if self.config.schema == "source":
                example = {
                    "id": id,
                    "speaker_id": spk_id,
                    "path": os.path.join(filepath["speech"]["SPK00_" + spk_group + "00"], id + ".wav"),
                    "audio": os.path.join(filepath["speech"]["SPK00_" + spk_group + "00"], id + ".wav"),
                    "text": transcripts[id],
                    "gender": gender,
                }
                yield key, example
            elif self.config.schema == "seacrowd_sptext":
                example = {
                    "id": str(id),
                    "speaker_id": spk_id,
                    "path": os.path.join(filepath["speech"]["SPK00_" + spk_group + "00"], id + ".wav"),
                    "audio": os.path.join(filepath["speech"]["SPK00_" + spk_group + "00"], id + ".wav"),
                    "text": transcripts[id],
                    "metadata": {
                        "speaker_age": None,
                        "speaker_gender": gender,
                    },
                }
                yield key, example



from pathlib import Path
from typing import List

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import (
    DEFAULT_SEACROWD_VIEW_NAME,
    DEFAULT_SOURCE_VIEW_NAME,
    Tasks,
)

_DATASETNAME = "parallel_id_nyo"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LOCAL = False
_LANGUAGES = ["ind", "abl"]

_CITATION = """\
@article{Abidin_2021,
doi = {10.1088/1742-6596/1751/1/012036},
url = {https://dx.doi.org/10.1088/1742-6596/1751/1/012036},
year = {2021},
month = {jan},
publisher = {IOP Publishing},
volume = {1751},
number = {1},
pages = {012036},
author = {Z Abidin and  Permata and I Ahmad and  Rusliyawati},
title = {Effect of mono corpus quantity on statistical machine translation
Indonesian - Lampung dialect of nyo},
journal = {Journal of Physics: Conference Series},
abstract = {Lampung Province is located on the island of Sumatera. For the
immigrants in Lampung, they have difficulty in
communicating with the indigenous people of Lampung. As an alternative, both
immigrants and the indigenous people of Lampung speak Indonesian.
This research aims to build a language model from Indonesian language and a
 translation model from the Lampung language dialect of nyo, both models will
 be combined in a Moses decoder.
This research focuses on observing the effect of adding mono corpus to the
 experimental statistical machine translation of
 Indonesian - Lampung dialect of nyo.
This research uses 3000 pair parallel corpus in Indonesia language and
Lampung language dialect of nyo as source language
and uses 3000 mono corpus sentences in Lampung language
dialect of nyo as target language. The results showed that the accuracy
value in bilingual evalution under-study score when using 1000 sentences,
2000 sentences, 3000 sentences mono corpus
show the accuracy value of the bilingual evaluation under-study,
respectively, namely 40.97 %, 41.80 % and 45.26 %.}
}
"""

_DESCRIPTION = """\
Dataset that contains Indonesian - Lampung language pairs.

The original data should contains 3000 rows, unfortunately,
not all of the instances in the original data is aligned perfectly.
Thus, this data only have the aligned ones, which only contain 1727 pairs.
"""

_HOMEPAGE = "https://drive.google.com/drive/folders/1oNpybrq5OJ_4Ne0HS5w9eHqnZlZASpmC?usp=sharing"

_LICENSE = "Unknown"

# WARNING: Incomplete data!
_URLs = {
    "train": "https://raw.githubusercontent.com/haryoa/IndoData/main/data_ind_lampung_1729_line.csv"
}

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"

COL_INDONESIA = "indo"
COL_LAMPUNG = "lampung"


class ParallelIdNyo(datasets.GeneratorBasedBuilder):
    """Dataset that contains Indonesian - Lampung language pairs."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="parallel_id_nyo_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="Parallel Id-Nyo source schema",
            schema="source",
            subset_id="parallel_id_nyo",
        ),
        SEACrowdConfig(
            name="parallel_id_nyo_seacrowd_t2t",
            version=datasets.Version(_SEACROWD_VERSION),
            description="Parallel Id-Nyo Nusantara schema",
            schema="seacrowd_t2t",
            subset_id="parallel_id_nyo",
        ),
    ]

    DEFAULT_CONFIG_NAME = "ted_en_id_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "label": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(
        self, dl_manager: datasets.DownloadManager
    ) -> List[datasets.SplitGenerator]:
        path = Path(dl_manager.download_and_extract(_URLs["train"]))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": path},
            )
        ]

    def _generate_examples(self, filepath: Path):

        df = pd.read_csv(filepath).reset_index()

        if self.config.schema == "source":
            for idx, row in df.iterrows():
                ex = {
                    "id": str(idx),
                    "text": str(row[COL_INDONESIA]).rstrip(),
                    "label": str(row[COL_LAMPUNG]).rstrip(),
                }
                yield idx, ex
        elif self.config.schema == "seacrowd_t2t":
            for idx, row in df.iterrows():
                ex = {
                    "id": str(idx),
                    "text_1": str(row[COL_INDONESIA]).rstrip(),
                    "text_2": str(row[COL_LAMPUNG]).rstrip(),
                    "text_1_name": "ind",
                    "text_2_name": "abl",  # code name for lampung Nyo
                }
                yield idx, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")


if __name__ == "__main__":
    datasets.load_dataset(__file__)
